@@ -108,9 +108,6 @@
 #include "agent_module_includes.h"
 #include "mib_module_includes.h"
 
-char            dontReadConfigFiles;
-char           *optconfigfile;
-
 #ifdef HAVE_UNISTD_H
 void
 snmpd_set_agent_user(const char *token, char *cptr)
@@ -186,9 +183,9 @@ snmpd_set_agent_address(const char *token, char *cptr)
         /*
          * append to the older specification string 
          */
-        sprintf(buf, "%s,%s", ptr, cptr);
+        snprintf(buf, SPRINT_MAX_LEN, "%s,%s", ptr, cptr);
     } else {
-        strcpy(buf, cptr);
+        strncpy(buf, cptr, SPRINT_MAX_LEN);
     }
 
     DEBUGMSGTL(("snmpd_ports", "port spec: %s\n", buf));
@@ -224,14 +221,17 @@ init_agent_read_config(const char *app)
 #endif
 #ifndef NETSNMP_DISABLE_SNMPV2C
         register_app_config_handler("trap2sink",
-                                    snmpd_parse_config_trap2sink, NULL,
+                                    snmpd_parse_config_trap2sink, 
+                                    snmpd_free_trapsinks,
                                     "host [community] [port]");
         register_app_config_handler("informsink",
-                                    snmpd_parse_config_informsink, NULL,
+                                    snmpd_parse_config_informsink,
+                                    snmpd_free_trapsinks,
                                     "host [community] [port]");
 #endif
         register_app_config_handler("trapsess",
-                                    snmpd_parse_config_trapsess, NULL,
+                                    snmpd_parse_config_trapsess,
+                                    snmpd_free_trapsinks,
                                     "[snmpcmdargs] host");
     }
 #if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
@@ -279,7 +279,9 @@ update_config(void)
     snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                         SNMPD_CALLBACK_PRE_UPDATE_CONFIG, NULL);
     free_config();
+#ifdef USING_MIBII_VACM_CONF_MODULE
     vacm_standard_views(0,0,NULL,NULL);
+#endif
     read_configs();
 }
 
