@@ -1671,17 +1671,14 @@ shutdown_tree(void) {
 void
 clear_subtree (netsnmp_subtree *sub) {
 
-    netsnmp_subtree *nxt;
+    netsnmp_subtree *c;
     
     if (sub == NULL)
 	return;
 
-    for(nxt = sub; nxt;) {
-        if (nxt->children != NULL) {
-            clear_subtree(nxt->children);
-        }
-        sub = nxt;
-        nxt = nxt->next;
+    for(c = sub; c;) {
+        sub = c;
+        c = c->children;
         netsnmp_subtree_free(sub);
     }
 
@@ -1706,6 +1703,7 @@ void
 clear_context(void) {
 
     subtree_context_cache *ptr = NULL, *next = NULL;
+    netsnmp_subtree *t, *u;
 
     DEBUGMSGTL(("agent_registry", "clear context\n"));
 
@@ -1713,8 +1711,9 @@ clear_context(void) {
     while (ptr) {
 	next = ptr->next;
 
-	if (ptr->first_subtree) {
-	    clear_subtree(ptr->first_subtree);
+	for (t = ptr->first_subtree; t; t = u) {
+            u = t->next;
+	    clear_subtree(t);
 	}
 
 	SNMP_FREE(ptr->context_name);
