@@ -254,7 +254,7 @@ create_trap_session2(const char *sink, const char* sinkport,
     if ((NULL == netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID,
                                        NETSNMP_DS_LIB_CLIENT_ADDR)) && 
         ((0 == strcmp("localhost",sink)) || (0 == strcmp("127.0.0.1",sink))))
-        session.localname = "localhost";
+        session.localname = strdup("localhost");
 
     t = netsnmp_tdomain_transport_full("snmptrap", sink, 0, NULL, sinkport);
     if (t != NULL) {
@@ -433,7 +433,7 @@ convert_v2pdu_to_v1( netsnmp_pdu* template_v2pdu )
         len = vblist->val_len / sizeof(oid);
         if ( len <= 2 ) {
             snmp_log(LOG_WARNING,
-                     "send_trap: v2 trapOID too short (%d)\n", len);
+                     "send_trap: v2 trapOID too short (%d)\n", (int)len);
             snmp_free_pdu(template_v1pdu);
             return NULL;
         }
@@ -607,7 +607,7 @@ convert_v1pdu_to_v2( netsnmp_pdu* template_v1pdu )
  *			
  * @param specific is the specific trap value.
  *
- * @param enterprise is an enterprise oid in which you want to send specifc 
+ * @param enterprise is an enterprise oid in which you want to send specific 
  *	traps from. 
  *
  * @param enterprise_length is the length of the enterprise oid, use macro,
@@ -896,7 +896,7 @@ send_trap_to_sess(netsnmp_session * sess, netsnmp_pdu *template_pdu)
     if (!sess || !template_pdu)
         return;
 
-    DEBUGMSGTL(("trap", "sending trap type=%d, version=%d\n",
+    DEBUGMSGTL(("trap", "sending trap type=%d, version=%ld\n",
                 template_pdu->command, sess->version));
 
 #ifndef NETSNMP_DISABLE_SNMPV1
@@ -922,7 +922,7 @@ send_trap_to_sess(netsnmp_session * sess, netsnmp_pdu *template_pdu)
     } else {
         if ((sess->version == SNMP_VERSION_3) &&
                 (pdu->command == SNMP_MSG_TRAP2) &&
-                (pdu->securityEngineIDLen == 0)) {
+                (sess->securityEngineIDLen == 0)) {
             len = snmpv3_get_engineID(tmp, sizeof(tmp));
             memdup(&pdu->securityEngineID, tmp, len);
             pdu->securityEngineIDLen = len;
@@ -1101,7 +1101,6 @@ snmpd_parse_config_trap2sink(const char *word, char *cptr)
 {
     char            tmpbuf[1024];
     char           *sp, *cp, *pp = NULL;
-    int             sinkport;
     char            *st;
 
     if (!snmp_trapcommunity)
@@ -1124,7 +1123,6 @@ snmpd_parse_config_informsink(const char *word, char *cptr)
 {
     char            tmpbuf[1024];
     char           *sp, *cp, *pp = NULL;
-    int             sinkport;
     char            *st;
 
     if (!snmp_trapcommunity)
