@@ -11,10 +11,6 @@
 
 #include <net-snmp/agent/cache_handler.h>
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 static netsnmp_cache  *cache_head = NULL;
 static int             cache_outstanding_valid = 0;
 static int             _cache_load( netsnmp_cache *cache );
@@ -197,6 +193,10 @@ netsnmp_cache_timer_start(netsnmp_cache *cache)
     if(NULL == cache)
         return 0;
 
+    DEBUGMSGTL(( "cache_timer:start", "OID: "));
+    DEBUGMSGOID(("cache_timer:start", cache->rootoid, cache->rootoid_len));
+    DEBUGMSG((   "cache_timer:start", "\n"));
+
     if(0 != cache->timer_id) {
         snmp_log(LOG_WARNING, "cache has existing timer id.\n");
         return cache->timer_id;
@@ -215,6 +215,7 @@ netsnmp_cache_timer_start(netsnmp_cache *cache)
         return 0;
     }
 
+    cache->flags &= ~NETSNMP_CACHE_AUTO_RELOAD;
     DEBUGMSGT(("cache_timer:start",
                "starting timer %d for cache %p\n", cache->timer_id, cache));
     return cache->timer_id;
@@ -236,6 +237,7 @@ netsnmp_cache_timer_stop(netsnmp_cache *cache)
                "stopping timer %d for cache %p\n", cache->timer_id, cache));
 
     snmp_alarm_unregister(cache->timer_id);
+    cache->flags |= NETSNMP_CACHE_AUTO_RELOAD;
 }
 
 
@@ -383,14 +385,14 @@ int
 netsnmp_cache_check_and_reload(netsnmp_cache * cache)
 {
     if (!cache) {
-        DEBUGMSG(("helper:cache_handler", " no cache\n"));
+        DEBUGMSGT(("helper:cache_handler", " no cache\n"));
         return 0;	/* ?? or -1 */
     }
     if (!cache->valid || netsnmp_cache_check_expired(cache))
         return _cache_load( cache );
     else {
-        DEBUGMSG(("helper:cache_handler", " cached (%d)\n",
-                  cache->timeout));
+        DEBUGMSGT(("helper:cache_handler", " cached (%d)\n",
+                   cache->timeout));
         return 0;
     }
 }
@@ -420,12 +422,16 @@ netsnmp_cache_helper_handler(netsnmp_mib_handler * handler,
                              netsnmp_agent_request_info * reqinfo,
                              netsnmp_request_info * requests)
 {
+    char addrstr[32];
+
     netsnmp_cache  *cache = NULL;
+    netsnmp_handler_args cache_hint;
 
     DEBUGMSGTL(("helper:cache_handler", "Got request (%d) for %s: ",
                 reqinfo->mode, reginfo->handlerName));
     DEBUGMSGOID(("helper:cache_handler", reginfo->rootoid,
                  reginfo->rootoid_len));
+    DEBUGMSG(("helper:cache_handler", "\n"));
 
     netsnmp_assert(handler->flags & MIB_HANDLER_AUTO_NEXT);
 
@@ -433,10 +439,24 @@ netsnmp_cache_helper_handler(netsnmp_mib_handler * handler,
     if (netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
                                NETSNMP_DS_AGENT_NO_CACHING) ||
         !cache || !cache->enabled || !cache->load_cache) {
-        DEBUGMSG(("helper:cache_handler", " caching disabled or "
-                  "cache not found, disabled or had no load method\n"));
+        DEBUGMSGT(("helper:cache_handler", " caching disabled or "
+                   "cache not found, disabled or had no load method\n"));
         return SNMP_ERR_NOERROR;
     }
+    snprintf(addrstr,sizeof(addrstr), "%p", cache);
+    DEBUGMSGTL(("helper:cache_handler", "using cache %s: ", addrstr));
+    DEBUGMSGOID(("helper:cache_handler", cache->rootoid, cache->rootoid_len));
+    DEBUGMSG(("helper:cache_handler", "\n"));
+
+    /*
+     * Make the handler-chain parameters available to
+     * the cache_load hook routine.
+     */
+    cache_hint.handler = handler;
+    cache_hint.reginfo = reginfo;
+    cache_hint.reqinfo = reqinfo;
+    cache_hint.requests = requests;
+    cache->cache_hint = &cache_hint;
 
     switch (reqinfo->mode) {
 
@@ -444,7 +464,6 @@ netsnmp_cache_helper_handler(netsnmp_mib_handler * handler,
     case MODE_GETNEXT:
     case MODE_GETBULK:
     case MODE_SET_RESERVE1: {
-        netsnmp_handler_args cache_hint;
 
         /*
          * only touch cache once per pdu request, to prevent a cache
@@ -454,25 +473,15 @@ netsnmp_cache_helper_handler(netsnmp_mib_handler * handler,
          * a previous (delegated) request is still using the cache.
          * maybe use a reference counter?
          */
-        if (netsnmp_cache_is_valid(reqinfo, reginfo->handlerName))
+        if (netsnmp_cache_is_valid(reqinfo, addrstr))
             return SNMP_ERR_NOERROR;
 
-        if (cache->flags & NETSNMP_CACHE_HINT_HANDLER_ARGS) {
-            netsnmp_assert(NULL == cache->cache_hint);
-            cache_hint.handler = handler;
-            cache_hint.reginfo = reginfo;
-            cache_hint.reqinfo = reqinfo;
-            cache_hint.requests = requests;
-            cache->cache_hint = &cache_hint;
-        }
-
         /*
          * call the load hook, and update the cache timestamp.
          * If it's not already there, add to reqinfo
          */
         netsnmp_cache_check_and_reload(cache);
-        netsnmp_cache_reqinfo_insert(cache, reqinfo, reginfo->handlerName);
-        cache->cache_hint = NULL;
+        netsnmp_cache_reqinfo_insert(cache, reqinfo, addrstr);
         /** next handler called automatically - 'AUTO_NEXT' */
         }
         return SNMP_ERR_NOERROR;
@@ -481,7 +490,7 @@ netsnmp_cache_helper_handler(netsnmp_mib_handler * handler,
     case MODE_SET_FREE:
     case MODE_SET_ACTION:
     case MODE_SET_UNDO:
-        netsnmp_assert(netsnmp_cache_is_valid(reqinfo, reginfo->handlerName));
+        netsnmp_assert(netsnmp_cache_is_valid(reqinfo, addrstr));
         /** next handler called automatically - 'AUTO_NEXT' */
         return SNMP_ERR_NOERROR;
 
@@ -502,11 +511,10 @@ netsnmp_cache_helper_handler(netsnmp_mib_handler * handler,
     default:
         snmp_log(LOG_WARNING, "cache_handler: Unrecognised mode (%d)\n",
                  reqinfo->mode);
-        netsnmp_set_all_requests_error(reqinfo, requests,
-                                       SNMP_ERR_GENERR);
+        netsnmp_request_set_error_all(requests, SNMP_ERR_GENERR);
         return SNMP_ERR_GENERR;
     }
-    netsnmp_set_all_requests_error(reqinfo, requests, SNMP_ERR_GENERR);
+    netsnmp_request_set_error_all(requests, SNMP_ERR_GENERR);
     return SNMP_ERR_GENERR;     /* should never get here */
 }
 
@@ -534,8 +542,7 @@ _cache_load( netsnmp_cache *cache )
     if ( cache->load_cache)
         ret = cache->load_cache(cache, cache->magic);
     if (ret < 0) {
-        DEBUGMSG(("helper:cache_handler", " load failed (%d)\n",
-                  ret));
+        DEBUGMSGT(("helper:cache_handler", " load failed (%d)\n", ret));
         cache->valid = 0;
         return ret;
     }
@@ -556,8 +563,7 @@ _cache_load( netsnmp_cache *cache )
         atime_setMarker(cache->timestamp);
     else
         cache->timestamp = atime_newMarker();
-    DEBUGMSG(("helper:cache_handler", " loaded (%d)\n",
-              cache->timeout));
+    DEBUGMSGT(("helper:cache_handler", " loaded (%d)\n", cache->timeout));
 
     return ret;
 }
