@@ -15,10 +15,6 @@
 #include <net-snmp/agent/serialize.h>
 #include <net-snmp/agent/read_only.h>
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 /** @defgroup scalar_group_group scalar_group
  *  Process groups of scalars.
  *  @ingroup leaf
@@ -161,7 +157,9 @@ netsnmp_scalar_group_helper_handler(netsnmp_mib_handler *handler,
 	/*
 	 * ... always assuming this is (potentially) valid, of course.
 	 */
-	if (subid > sgroup->ubound)
+        if (subid < sgroup->lbound)
+            subid = sgroup->lbound;
+	else if (subid > sgroup->ubound)
             return SNMP_ERR_NOERROR;
         
         root_tmp[reginfo->rootoid_len++] = subid;
