@@ -84,6 +84,25 @@ netsnmp_create_table_data_set(const char *table_name)
     return table_set;
 }
 
+void netsnmp_delete_table_data_set(netsnmp_table_data_set *table_set)
+{
+    netsnmp_table_data_set_storage *ptr, *next;
+    netsnmp_table_row *prow, *pnextrow;
+
+    for (ptr = table_set->default_row; ptr; ptr = next) {
+        next = ptr->next;
+        free(ptr);
+    }
+    table_set->default_row = NULL;
+    for (prow = table_set->table->first_row; prow; prow = pnextrow) {
+        pnextrow = prow->next;
+        netsnmp_table_dataset_remove_and_delete_row(table_set, prow);
+    }
+    table_set->table->first_row = NULL;
+    netsnmp_table_data_delete_table(table_set->table);
+    free(table_set);
+}
+
 /** clones a dataset row, including all data. */
 netsnmp_table_row *
 netsnmp_table_data_set_clone_row(netsnmp_table_row *row)
@@ -574,7 +593,13 @@ netsnmp_table_data_set_helper_handler(netsnmp_mib_handler *handler,
         case MODE_GET:
         case MODE_GETNEXT:
         case MODE_GETBULK:     /* XXXWWW */
-            if (data && data->data.voidp)
+            if (!data || data->type == SNMP_NOSUCHINSTANCE)
+                netsnmp_set_request_error(reqinfo, request, SNMP_NOSUCHINSTANCE);
+            else
+                /*
+                 * Note: data->data.voidp can be NULL, e.g. when a zero-length
+                 * octet string has been stored in the table cache.
+                 */
                 netsnmp_table_data_build_result(reginfo, reqinfo, request,
                                                 row,
                                                 table_info->colnum,
