@@ -12,13 +12,16 @@ config_require(hardware/cpu/cpu_pcp)
 #elif defined(irix6)
 config_require(hardware/cpu/cpu_sysinfo)
 
-#elif (defined(netbsd) || defined(netbsd1) || defined(netbsdelf) || defined(netbsdelf2)|| defined(netbsdelf3) || defined(openbsd2)|| defined(openbsd3) || defined(openbsd4))
+#elif defined(dragonfly)
+config_require(hardware/cpu/cpu_kinfo)
+
+#elif (defined(netbsd) || defined(netbsd1) || defined(netbsdelf) || defined(netbsdelf2)|| defined(netbsdelf3) || defined(openbsd2) || defined(openbsd3) || defined(openbsd4) || defined(freebsd4)  || defined(freebsd5) || defined(freebsd6))
 config_require(hardware/cpu/cpu_sysctl)
 
-#elif (defined(freebsd2) || defined(freebsd3) || defined(freebsd4)  || defined(freebsd5)|| defined(freebsd6))
+#elif (defined(freebsd2) || defined(freebsd3))
 config_require(hardware/cpu/cpu_nlist)
 
-#elif (defined(aix4) || defined(aix5))
+#elif (defined(aix4) || defined(aix5) || defined(aix6))
 config_require(hardware/cpu/cpu_perfstat)
 
 #elif (defined(solaris2))
