@@ -37,7 +37,8 @@ void _cpu_copy_stats( netsnmp_cpu_info *cpu );
      *   (including descriptions)
      */
 void init_cpu_sysctl( void ) {
-    int               i, n;
+    int               n;
+    size_t            i;
     int               ncpu_mib[]  = { CTL_HW, HW_NCPU };
     int               model_mib[] = { CTL_HW, HW_MODEL };
     char              descr[ SNMP_MAXBUF ];
@@ -45,11 +46,15 @@ void init_cpu_sysctl( void ) {
     strcpy(cpu->name, "Overall CPU statistics");
 
     i = sizeof(n);
-    sysctl(ncpu_mib, 2, &n, (void *)&i, NULL, 0);
+    sysctl(ncpu_mib, 2, &n, &i, NULL, 0);
     if ( n <= 0 )
         n = 1;   /* Single CPU system */
     i = sizeof(descr);
-    sysctl(model_mib, 2, descr, (void *)&i, NULL, 0);
+#if defined(__NetBSD__) && ( defined(__i386__) || defined(__x86_64__) )
+    sysctlbyname("machdep.cpu_brand", descr, (void *)&i, NULL, 0);
+#else
+    sysctl(model_mib, 2, descr, &i, NULL, 0);
+#endif
     for ( i = 0; i < n; i++ ) {
         cpu = netsnmp_cpu_get_byIdx( i, 1 );
         cpu->status = 2;  /* running */
@@ -60,18 +65,18 @@ void init_cpu_sysctl( void ) {
 }
 
 
+#if defined(__NetBSD__)
+#define NETSNMP_CPU_STATS uint64_t
+#else
 #define NETSNMP_CPU_STATS long
-#if defined(KERN_CPUSTATS)                /* BSDi */
+#endif
+
+#if defined(__NetBSD__)
+#define NETSNMP_KERN_CPU  KERN_CP_TIME
+#elif defined(KERN_CPUSTATS)              /* BSDi */
 #define NETSNMP_KERN_CPU  KERN_CPUSTATS
 #elif defined(KERN_CPTIME)                /* OpenBSD */
 #define NETSNMP_KERN_CPU  KERN_CPTIME
-#elif defined(KERN_CP_TIME)               /* NetBSD */
-#define NETSNMP_KERN_CPU  KERN_CP_TIME
-
-#if defined(netbsdelf3)
-#undef  NETSNMP_CPU_STATS 
-#define NETSNMP_CPU_STATS uint64_t
-#endif
 
 #elif defined(__FreeBSD__)
 #define NETSNMP_KERN_CPU  0    /* dummy value - sysctlnametomib(2) should be used */
@@ -152,14 +157,14 @@ int netsnmp_cpu_arch_load( netsnmp_cache *cache, void *magic ) {
 #ifdef NETSNMP_KERN_MCPU 
     NETSNMP_KERN_MCPU_TYPE *mcpu_stats;
     int            mcpu_mib[] = { CTL_KERN, NETSNMP_KERN_MCPU };
-    size_t         mcpu_size  = sizeof(NETSNMP_KERN_MCPU_TYPE);
+    size_t         mcpu_size;
 #endif
     NETSNMP_VM_STATS_TYPE mem_stats;
     int            mem_mib[] = { CTL_VM, NETSNMP_VM_STATS };
     size_t         mem_size  = sizeof(NETSNMP_VM_STATS_TYPE);
     netsnmp_cpu_info *cpu = netsnmp_cpu_get_byIdx( -1, 0 );
 
-#if defined(__FreeBSD__)
+#if (defined(__FreeBSD__) || defined(__NetBSD__))
     sysctlbyname("kern.cp_time", cpu_stats, &cpu_size, NULL, 0);
 #else
     sysctl(cpu_mib, 2,  cpu_stats, &cpu_size, NULL, 0);
@@ -189,9 +194,9 @@ int netsnmp_cpu_arch_load( netsnmp_cache *cache, void *magic ) {
 #endif
 
 #ifdef NETSNMP_KERN_MCPU
-    mcpu_stats = malloc(cpu_num*sizeof(NETSNMP_KERN_MCPU_TYPE));
-    sysctl(mcpu_mib, 2, mcpu_stats,
-           cpu_num*sizeof(NETSNMP_KERN_MCPU_TYPE), NULL, 0);
+    mcpu_size  = cpu_num*sizeof(NETSNMP_KERN_MCPU_TYPE);
+    mcpu_stats = malloc(mcpu_size);
+    sysctl(mcpu_mib, 2, mcpu_stats, &mcpu_size, NULL, 0);
     for ( i = 0; i < cpu_num; i++ ) {
         cpu = netsnmp_cpu_get_byIdx( i, 0 );
         /* XXX - per-CPU statistics - mcpu_mib[i].??? */
