@@ -1,7 +1,7 @@
 /*
  * interface data access header
  *
- * $Id: interface.h,v 1.16 2006/09/26 23:34:50 rstory Exp $
+ * $Id: interface.h 19862 2011-01-13 12:04:53Z dts12 $
  */
 #ifndef NETSNMP_ACCESS_INTERFACE_CONFIG_H
 #define NETSNMP_ACCESS_INTERFACE_CONFIG_H
@@ -36,7 +36,8 @@ config_exclude(mibII/interfaces)
     config_require(if-mib/data_access/interface_ioctl)
 
 #   elif defined( openbsd3 ) || defined( openbsd4 ) || \
-    defined( freebsd4 ) || defined( freebsd5 ) || defined( freebsd6 )
+         defined( freebsd4 ) || defined( freebsd5 ) || defined( freebsd6 ) || \
+         defined (darwin)    || defined( dragonfly )
 
     config_require(if-mib/data_access/interface_sysctl)
 
