@@ -1,7 +1,7 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: interface_linux.c 18100 2010-02-05 09:48:46Z jsafranek $
+ * $Id: interface_linux.c 19915 2011-02-08 17:37:01Z hardaker $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -345,7 +345,7 @@ _parse_stats(netsnmp_interface_entry *entry, char *stats, int expected)
 
     if ((*stats == 'N') &&
         (0 == strncmp(stats, "No statistics available",
-                      sizeof("No statistics available"))))
+                      strlen("No statistics available"))))
         return -1;
 
     /*
