@@ -1,7 +1,7 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: ipaddress_ioctl.c 17850 2009-11-30 16:30:12Z dts12 $
+ * $Id: ipaddress_ioctl.c 20445 2011-05-22 19:34:10Z dts12 $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -232,7 +232,7 @@ _netsnmp_ioctl_ipaddress_container_load_v4(netsnmp_container *container,
         netsnmp_assert(AF_INET == ifrp->ifr_addr.sa_family);
         si = (struct sockaddr_in *) &ifrp->ifr_addr;
         entry->ia_prefix_len =
-            netsnmp_ipaddress_ipv4_prefix_len(si->sin_addr.s_addr);
+            netsnmp_ipaddress_ipv4_prefix_len(ntohl(si->sin_addr.s_addr));
 
         /*
          * get flags
