@@ -1,7 +1,7 @@
 /*
  *  Systemstats MIB architecture support
  *
- * $Id: systemstats_common.c 17714 2009-08-04 10:57:52Z jsafranek $
+ * $Id: systemstats_common.c 19907 2011-02-01 09:41:28Z jsafranek $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -252,6 +252,7 @@ netsnmp_access_systemstats_entry_update_stats(netsnmp_systemstats_entry * prev_v
         if (NULL == prev_vals->old_stats) {
             return -2;
         }
+        memcpy(&prev_vals->stats, &new_vals->stats, sizeof(new_vals->stats));
     }
     else {
         /*
@@ -262,7 +263,7 @@ netsnmp_access_systemstats_entry_update_stats(netsnmp_systemstats_entry * prev_v
         prev_vals->stats.InAddrErrors = new_vals->stats.InAddrErrors;
         prev_vals->stats.InUnknownProtos = new_vals->stats.InUnknownProtos;
         prev_vals->stats.InTruncatedPkts = new_vals->stats.InTruncatedPkts;
-        prev_vals->stats.ReasmReqds = new_vals->stats.InTruncatedPkts;
+        prev_vals->stats.ReasmReqds = new_vals->stats.ReasmReqds;
         prev_vals->stats.ReasmOKs = new_vals->stats.ReasmOKs;
         prev_vals->stats.ReasmFails = new_vals->stats.ReasmFails;
         prev_vals->stats.InDiscards = new_vals->stats.InDiscards;
