@@ -1,7 +1,7 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: systemstats_linux.c,v 1.10.2.1 2006/03/09 05:25:44 hardaker Exp $
+ * $Id: systemstats_linux.c 19699 2010-11-30 04:58:34Z hardaker $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -11,7 +11,7 @@
 #include <net-snmp/data_access/systemstats.h>
 
 static int _systemstats_v4(netsnmp_container* container, u_int load_flags);
-#if defined (INET6)
+#if defined (NETSNMP_ENABLE_IPV6)
 static int _systemstats_v6(netsnmp_container* container, u_int load_flags);
 #endif
 
@@ -54,7 +54,7 @@ netsnmp_access_systemstats_container_arch_load(netsnmp_container* container,
                                              u_int load_flags)
 {
     int rc1;
-#if defined (INET6)
+#if defined (NETSNMP_ENABLE_IPV6)
     int rc2;
 #endif
 
@@ -70,7 +70,7 @@ netsnmp_access_systemstats_container_arch_load(netsnmp_container* container,
      * would have logged its own message.
      */
     rc1 = _systemstats_v4(container, load_flags);
-#if defined (INET6)
+#if defined (NETSNMP_ENABLE_IPV6)
     rc2 = _systemstats_v6(container, load_flags);
     if ((rc1 == rc2) || (rc1 < rc2))
         return rc1;
@@ -206,12 +206,17 @@ _systemstats_v4(netsnmp_container* container, u_int load_flags)
         /*
          * add to container
          */
-        CONTAINER_INSERT(container, entry);
+        if (CONTAINER_INSERT(container, entry) < 0)
+        {
+            DEBUGMSGTL(("access:systemstats:container","error with systemstats_entry: insert into container failed.\n"));
+            netsnmp_access_systemstats_entry_free(entry);
+        }
     }
+
     return 0;
 }
     
-#if defined (INET6)
+#if defined (NETSNMP_ENABLE_IPV6)
 static int
 _systemstats_v6(netsnmp_container* container, u_int load_flags)
 {
@@ -369,8 +374,13 @@ _systemstats_v6(netsnmp_container* container, u_int load_flags)
     /*
      * add to container
      */
-    CONTAINER_INSERT(container, entry);
+        if (CONTAINER_INSERT(container, entry) < 0)
+        {
+            DEBUGMSGTL(("access:systemstats:container","error with systemstats_entry: insert into container failed.\n"));
+            netsnmp_access_systemstats_entry_free(entry);
+        }
+
 
     return rc;
 }
-#endif /* INET6 */
+#endif /* NETSNMP_ENABLE_IPV6 */
