@@ -145,7 +145,7 @@
 
 #if !defined (WIN32) && !defined (cygwin)
 
-#if !defined(CAN_USE_SYSCTL) || !defined(IPCTL_STATS)
+#if !defined(NETSNMP_CAN_USE_SYSCTL) || !defined(IPCTL_STATS)
 #ifndef solaris2
 
 #if defined(freebsd2) || defined(hpux11) || defined(linux)
@@ -202,6 +202,7 @@ var_ipAddrEntry(struct variable *vp,
 #endif
     static struct ifnet ifnet;
 #endif                          /* hpux11 */
+    static in_addr_t	addr_ret;
 
     /*
      * fill in object part of name for current (less sizeof instance part) 
@@ -293,11 +294,11 @@ var_ipAddrEntry(struct variable *vp,
     *var_len = sizeof(long_return);
     switch (vp->magic) {
     case IPADADDR:
+    	*var_len = sizeof(addr_ret);
 #ifdef hpux11
-        long_return = lowin_ifaddr.Addr;
-        return (u_char *) & long_return;
+        addr_ret = lowin_ifaddr.Addr;
+        return (u_char *) & addr_ret;
 #elif defined(linux) || defined(sunV3)
-        *var_len = sizeof(((struct sockaddr_in *) &lowin_ifnet.if_addr)->sin_addr.s_addr);
         return (u_char *) & ((struct sockaddr_in *) &lowin_ifnet.if_addr)->
             sin_addr.s_addr;
 #else
@@ -308,16 +309,16 @@ var_ipAddrEntry(struct variable *vp,
         long_return = lowinterface;
         return (u_char *) & long_return;
     case IPADNETMASK:
+        *var_len = sizeof(addr_ret);
 #ifdef hpux11
-        long_return = lowin_ifaddr.NetMask;
-        return (u_char *) & long_return;
+        addr_ret = lowin_ifaddr.NetMask;
+        return (u_char *) & addr_ret;
 #elif defined(linux)
-        *var_len = sizeof(((struct sockaddr_in *) &lowin_ifnet.ia_subnetmask)->sin_addr.s_addr);
         return (u_char *) & ((struct sockaddr_in *) &lowin_ifnet.
                              ia_subnetmask)->sin_addr.s_addr;
 #elif !defined(sunV3)
-        long_return = lowin_ifaddr.ia_subnetmask;
-        return (u_char *) & long_return;
+        addr_ret = lowin_ifaddr.ia_subnetmask;
+        return (u_char *) & addr_ret;
 #endif
     case IPADBCASTADDR:
 #ifdef hpux11
@@ -341,7 +342,7 @@ var_ipAddrEntry(struct variable *vp,
 #ifdef hpux11
         long_return = lowin_ifaddr.ReasmMaxSize;
         return (u_char *) & long_return;
-#elif defined(NO_DUMMY_VALUES)
+#elif defined(NETSNMP_NO_DUMMY_VALUES)
         return NULL;
 #else
         long_return = -1;
@@ -355,13 +356,15 @@ var_ipAddrEntry(struct variable *vp,
 }
 
 #ifdef freebsd2
-static struct in_ifaddr *in_ifaddraddr;
+static struct in_ifaddr *in_ifaddraddr = NULL;
 
 static void
 Address_Scan_Init(void)
 {
-    auto_nlist(IFADDR_SYMBOL, (char *) &in_ifaddraddr,
+    int rc = auto_nlist(IFADDR_SYMBOL, (char *) &in_ifaddraddr,
                sizeof(in_ifaddraddr));
+    if (0 == rc)
+        in_ifaddraddr = NULL;
 }
 
 /*
@@ -380,8 +383,11 @@ Address_Scan_Next(Index, Retin_ifaddr)
         /*
          *      Get the "in_ifaddr" structure
          */
-        klookup((unsigned long) in_ifaddraddr, (char *) &in_ifaddr,
-                sizeof in_ifaddr);
+        if (!NETSNMP_KLOOKUP(in_ifaddraddr, (char *) &in_ifaddr, sizeof in_ifaddr)) {
+            DEBUGMSGTL(("mibII/ip:Address_Scan_Next", "klookup failed\n"));
+            return 0;
+        }
+
         in_ifaddraddr = in_ifaddr.ia_next;
 
         if (Retin_ifaddr)
@@ -394,8 +400,10 @@ Address_Scan_Next(Index, Retin_ifaddr)
 
         auto_nlist(IFNET_SYMBOL, (char *) &ifnetaddr, sizeof(ifnetaddr));
         while (ifnetaddr && ifnetaddr != in_ifaddr.ia_ifp) {
-            klookup((unsigned long) ifnetaddr, (char *) &ifnet,
-                    sizeof ifnet);
+            if (!NETSNMP_KLOOKUP(ifnetaddr, (char *) &ifnet, sizeof ifnet)) {
+                DEBUGMSGTL(("mibII/ip:Address_Scan_Next", "klookup failed\n"));
+                return 0;
+            }
             ifnetaddr = ifnet.if_next;
             iindex++;
         }
@@ -531,10 +539,7 @@ Address_Scan_Init(void)
  * NB: Index is the number of the corresponding interface, not of the address 
  */
 static int
-Address_Scan_Next(Index, Retifnet)
-    short          *Index;
-    struct ifnet   *Retifnet;
-
+Address_Scan_Next(short *Index, struct ifnet *Retifnet)
 {
     struct ifnet   ifnet_store;
     int fd;
@@ -624,7 +629,7 @@ var_ipAddrEntry(struct variable * vp,
     static mib2_ipAddrEntry_t Lowentry;
     int             Found = 0;
     req_e           req_type;
-    static uint32_t ipaddr_return;
+    static in_addr_t addr_ret;
 
     /*
      * fill in object part of name for current (less sizeof instance part) 
@@ -691,18 +696,25 @@ var_ipAddrEntry(struct variable * vp,
     *var_len = sizeof(long_return);
     switch (vp->magic) {
     case IPADADDR:
-	*var_len = sizeof(uint32_t);
-        ipaddr_return = Lowentry.ipAdEntAddr;
-        return (u_char *) & ipaddr_return;
+	*var_len = sizeof(addr_ret);
+        addr_ret = Lowentry.ipAdEntAddr;
+        return (u_char *) & addr_ret;
     case IPADIFINDEX:
+#ifdef NETSNMP_INCLUDE_IFTABLE_REWRITES
+        Lowentry.ipAdEntIfIndex.o_bytes[Lowentry.ipAdEntIfIndex.o_length] = '\0';
         long_return =
-            Interface_Index_By_Name(Lowentry.ipAdEntIfIndex.o_bytes,
-                                    Lowentry.ipAdEntIfIndex.o_length);
+            netsnmp_access_interface_index_find(Lowentry.
+                                                ipAdEntIfIndex.o_bytes);
+#else
+        long_return =
+           Interface_Index_By_Name(Lowentry.ipAdEntIfIndex.o_bytes,
+                                   Lowentry.ipAdEntIfIndex.o_length);
+#endif
         return (u_char *) & long_return;
     case IPADNETMASK:
-	*var_len = sizeof(uint32_t);
-        ipaddr_return = Lowentry.ipAdEntNetMask;
-        return (u_char *) & ipaddr_return;
+	*var_len = sizeof(addr_ret);
+        addr_ret = Lowentry.ipAdEntNetMask;
+        return (u_char *) & addr_ret;
     case IPADBCASTADDR:
 	long_return = Lowentry.ipAdEntBcastAddr;
 	return (u_char *) & long_return;
@@ -727,7 +739,7 @@ var_ipAddrEntry(struct variable * vp,
 	 *********************/
 
 
-#else                           /* CAN_USE_SYSCTL && IPCTL_STATS */
+#else                           /* NETSNMP_CAN_USE_SYSCTL && IPCTL_STATS */
 
 /*
  * Ideally, this would be combined with the code in interfaces.c.
@@ -864,7 +876,8 @@ var_ipAddrEntry(struct variable *vp,
     u_char         *cp;
     int             lowinterface = -1;
     int             i, interface;
-
+    static in_addr_t	addr_ret;
+    
     /*
      * fill in object part of name for current (less sizeof instance part) 
      */
@@ -913,23 +926,25 @@ var_ipAddrEntry(struct variable *vp,
     *var_len = sizeof(long_return);
     switch (vp->magic) {
     case IPADADDR:
-        long_return = ifs[i].addr.s_addr;
-        return (u_char *) & long_return;
+        *var_len = sizeof(addr_ret);
+        addr_ret = ifs[i].addr.s_addr;
+        return (u_char *) & addr_ret;
 
     case IPADIFINDEX:
         long_return = ifs[i].index;
         return (u_char *) & long_return;
 
     case IPADNETMASK:
-        long_return = ifs[i].mask.s_addr;
-        return (u_char *) & long_return;
+        *var_len = sizeof(addr_ret);
+        addr_ret = ifs[i].mask.s_addr;
+        return (u_char *) & addr_ret;
 
     case IPADBCASTADDR:
         long_return = ntohl(ifs[i].bcast.s_addr) & 1;
         return (u_char *) & long_return;
 
     case IPADREASMMAX:
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #else
         long_return = -1;
@@ -943,7 +958,7 @@ var_ipAddrEntry(struct variable *vp,
     return NULL;
 }
 
-#endif                          /* CAN_USE_SYSCTL && IPCTL_STATS */
+#endif                          /* NETSNMP_CAN_USE_SYSCTL && IPCTL_STATS */
 
 #else                           /* WIN32 cygwin */
 #include <iphlpapi.h>
@@ -967,7 +982,9 @@ var_ipAddrEntry(struct variable *vp,
     DWORD           status = NO_ERROR;
     DWORD           statusRetry = NO_ERROR;
     DWORD           dwActualSize = 0;
-
+    void           *result = NULL;
+    static in_addr_t 	addr_ret;
+    
     /*
      * fill in object part of name for current (less sizeof instance part) 
      */
@@ -1002,7 +1019,6 @@ var_ipAddrEntry(struct variable *vp,
                 }
             } else {
                 if (snmp_oid_compare(current, 14, name, *length) > 0) {
-
                     lowinterface = i;
                     memcpy(lowest, current, 14 * sizeof(oid));
                     break;      /* Since the table is sorted, no need to search further  */
@@ -1011,44 +1027,52 @@ var_ipAddrEntry(struct variable *vp,
         }
     }
 
-    if (lowinterface < 0) {
-        free(pIpAddrTable);
-        return NULL;
-    }
+    if (lowinterface < 0)
+        goto out;
+
     i = lowinterface;
     memcpy(name, lowest, 14 * sizeof(oid));
     *length = 14;
     *write_method = 0;
-    *var_len = sizeof(long_return);
     switch (vp->magic) {
     case IPADADDR:
-        long_return = pIpAddrTable->table[i].dwAddr;
-        return (u_char *) & long_return;
+        *var_len = sizeof(addr_ret);
+        addr_ret = pIpAddrTable->table[i].dwAddr;
+        result = &addr_ret;
+        break;
 
     case IPADIFINDEX:
+        *var_len = sizeof(long_return);
         long_return = pIpAddrTable->table[i].dwIndex;
-        free(pIpAddrTable);
-        return (u_char *) & long_return;
+        result = &long_return;
+        break;
 
     case IPADNETMASK:
-        long_return = pIpAddrTable->table[i].dwMask;
-        free(pIpAddrTable);
-        return (u_char *) & long_return;
+        *var_len = sizeof(addr_ret);
+        addr_ret = pIpAddrTable->table[i].dwMask;
+        result = &addr_ret;
+        break;
 
     case IPADBCASTADDR:
+        *var_len = sizeof(long_return);
         long_return = pIpAddrTable->table[i].dwBCastAddr;
-        free(pIpAddrTable);
-        return (u_char *) & long_return;
+        result = &long_return;
+        break;
 
     case IPADREASMMAX:
+        *var_len = sizeof(long_return);
         long_return = pIpAddrTable->table[i].dwReasmSize;
-        free(pIpAddrTable);
-        return (u_char *) & long_return;
+        result = &long_return;
+        break;
 
     default:
         DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ipAddrEntry\n",
                     vp->magic));
+        break;
     }
-    return NULL;
+
+out:
+    free(pIpAddrTable);
+    return result;
 }
 #endif                          /* WIN32 cygwin */
