@@ -63,14 +63,18 @@
 #if HAVE_SYS_QUEUE_H
 #include <sys/queue.h>
 #endif
+/* IRIX 6.5 build breaks on sys/socketvar.h because _KMEMUSER brings in 
+   sys/pda.h which doesn't compile */
+#ifndef irix6
 #if HAVE_SYS_SOCKETVAR_H
 #include <sys/socketvar.h>
 #endif
+#endif  /* irix6 */
 #if HAVE_NETINET_IP_VAR_H
 #include <netinet/ip_var.h>
 #endif
-#ifdef INET6
-#if HAVE_NETINET6_IP6_VAR_H
+#ifdef NETSNMP_ENABLE_IPV6
+#if HAVE_NETNETSNMP_ENABLE_IPV6_IP6_VAR_H
 #include <netinet6/ip6_var.h>
 #endif
 #endif
@@ -95,6 +99,9 @@
 #endif
 #ifdef linux
 #include "kernel_linux.h"
+#endif
+#ifdef NETBSD_STATS_VIA_SYSCTL
+#include "kernel_netbsd.h"
 #endif
 	/* or MIB_xxxCOUNTER_SYMBOL || hpux11 */
 #ifdef hpux
