@@ -37,7 +37,7 @@
 
 /** "include files" */
 #ifdef __lint
-# define SNMP_NO_DEBUGGING 1    /* keeps lint from complaining about the DEBUGMSG* macros */
+# define NETSNMP_NO_DEBUGGING 1    /* keeps lint from complaining about the DEBUGMSG* macros */
 #endif
 
 #include <net-snmp/net-snmp-config.h>
@@ -520,7 +520,7 @@ open_sendmailst(BOOL config)
 count_queuegroup(struct QGrp *qg)
 {
     struct QDir    *d;
-    char            cwd[200];
+    char            cwd[SNMP_MAXPATH];
     time_t          current_time = time(NULL);
 
     if (current_time <= (qg->last + dir_cache_time)) {
@@ -969,11 +969,13 @@ read_sendmailcf(BOOL config)
                 }
 
                 /*
-                 * skip to next , 
+                 * skip to one past the next , 
                  */
                 while (*p && *p != ',') {
                     p++;
                 }
+		if (*p)
+		    p++;
             }
 
             /*
