@@ -123,6 +123,12 @@ int                      tcp_estab = 0;
 #define TCP_STATS_CACHE_TIMEOUT	MIB_STATS_CACHE_TIMEOUT
 #endif
 
+#if defined(TCP_PORTS_IN_HOST_ORDER) && TCP_PORTS_IN_HOST_ORDER
+#define TCP_PORT_TO_HOST_ORDER(x) x
+#else
+#define TCP_PORT_TO_HOST_ORDER(x) ntohs(x)
+#endif
+
 oid             tcpTable_oid[] = { SNMP_OID_MIB2, 6, 13 };
 
 void
@@ -158,9 +164,9 @@ init_tcpTable(void)
     iinfo->get_first_data_point = tcpTable_first_entry;
     iinfo->get_next_data_point  = tcpTable_next_entry;
     iinfo->table_reginfo        = table_info;
-#if defined (WIN32) || defined (cygwin) || defined (solaris2)
+#if defined (WIN32) || defined (cygwin)
     iinfo->flags               |= NETSNMP_ITERATOR_FLAG_SORTED;
-#endif /* WIN32 || cygwin || solaris2 */
+#endif /* WIN32 || cygwin */
 
 
     /*
@@ -233,7 +239,7 @@ tcpTable_handler(netsnmp_mib_handler          *handler,
 #endif
                 break;
             case TCPCONNLOCALPORT:
-                port = ntohs((u_short)entry->TCPTABLE_LOCALPORT);
+                port = TCP_PORT_TO_HOST_ORDER((u_short)entry->TCPTABLE_LOCALPORT);
 	        snmp_set_var_typed_value(requestvb, ASN_INTEGER,
                                  (u_char *)&port, sizeof(port));
                 break;
@@ -249,7 +255,7 @@ tcpTable_handler(netsnmp_mib_handler          *handler,
 #endif
                 break;
             case TCPCONNREMOTEPORT:
-                port = ntohs((u_short)entry->TCPTABLE_REMOTEPORT);
+                port = TCP_PORT_TO_HOST_ORDER((u_short)entry->TCPTABLE_REMOTEPORT);
 	        snmp_set_var_typed_value(requestvb, ASN_INTEGER,
                                  (u_char *)&port, sizeof(port));
                 break;
@@ -279,6 +285,7 @@ tcpTable_handler(netsnmp_mib_handler          *handler,
 
 int
 TCP_Count_Connections( void ) {
+    tcpTable_load(NULL, NULL);
     return tcp_estab;
 }
 
@@ -338,7 +345,7 @@ tcpTable_next_entry( void **loop_context,
                                 sizeof(tcp_head[i].TCPTABLE_LOCALADDRESS));
 #endif
 
-    port = ntohs((u_short)tcp_head[i].TCPTABLE_LOCALPORT);
+    port = TCP_PORT_TO_HOST_ORDER((u_short)tcp_head[i].TCPTABLE_LOCALPORT);
     idx = idx->next_variable;
     snmp_set_var_value(idx, (u_char*)&port, sizeof(port));
 
@@ -352,7 +359,7 @@ tcpTable_next_entry( void **loop_context,
                                 sizeof(tcp_head[i].TCPTABLE_REMOTEADDRESS));
 #endif
 
-    port = ntohs((u_short)tcp_head[i].TCPTABLE_REMOTEPORT);
+    port = TCP_PORT_TO_HOST_ORDER((u_short)tcp_head[i].TCPTABLE_REMOTEPORT);
     idx = idx->next_variable;
     snmp_set_var_value(idx, (u_char*)&port, sizeof(port));
 
@@ -429,7 +436,7 @@ tcpTable_next_entry( void **loop_context,
 #endif
     snmp_set_var_value(idx, (u_char *)&addr, sizeof(addr));
 
-    port = ntohs(entry->TCPTABLE_LOCALPORT);
+    port = TCP_PORT_TO_HOST_ORDER(entry->TCPTABLE_LOCALPORT);
     idx = idx->next_variable;
     snmp_set_var_value(idx, (u_char*)&port, sizeof(port));
 
@@ -441,7 +448,7 @@ tcpTable_next_entry( void **loop_context,
 #endif
     snmp_set_var_value(idx, (u_char *)&addr, sizeof(addr));
 
-    port = ntohs(entry->TCPTABLE_REMOTEPORT);
+    port = TCP_PORT_TO_HOST_ORDER(entry->TCPTABLE_REMOTEPORT);
     idx = idx->next_variable;
     snmp_set_var_value(idx, (u_char*)&port, sizeof(port));
 
@@ -528,7 +535,7 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
     }
 
     if (tcp_size > 0) {
-        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table\n"));
+        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table (hpux11)\n"));
         return 0;
     }
     DEBUGMSGTL(("mibII/tcpTable", "Failed to load TCP Table (hpux11)\n"));
@@ -587,7 +594,7 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
 
     fclose(in);
 
-    DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table\n"));
+    DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table (linux)\n"));
     return 0;
 }
 #else                           /* linux */
@@ -624,6 +631,8 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
          * Note that since getMibstat returns rows in sorted order,
          *    we need to retain this order while building the list
          *    so new entries are added onto the end of the list.
+         * Note 2: at least Solaris 8-10 do not return rows in
+         *    sorted order anymore
          */
         nnew = SNMP_MALLOC_TYPEDEF(netsnmp_tcpConnEntry);
         if (nnew == NULL)
@@ -641,7 +650,7 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
     }
 
     if (tcp_head) {
-        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table\n"));
+        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table (solaris)\n"));
         return 0;
     }
     DEBUGMSGTL(("mibII/tcpTable", "Failed to load TCP Table (solaris)\n"));
@@ -674,7 +683,7 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
     if (status == NO_ERROR) {
         int           i;
 
-        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table\n"));
+        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table (WIN32)\n"));
         tcp_size = pTcpTable->dwNumEntries -1;  /* entries are counted starting with 0 */
         tcp_head = pTcpTable->table;
 
@@ -697,7 +706,7 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
 }
 #else                           /* WIN32 cygwin */
 
-#if (defined(CAN_USE_SYSCTL) && defined(TCPCTL_PCBLIST))
+#if (defined(NETSNMP_CAN_USE_SYSCTL) && defined(TCPCTL_PCBLIST))
 
 #if defined(freebsd4) || defined(darwin)
     #define NS_ELEM struct xtcpcb
@@ -711,7 +720,11 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
     size_t   len;
     int      sname[] = { CTL_NET, PF_INET, IPPROTO_TCP, TCPCTL_PCBLIST };
     char     *tcpcb_buf = NULL;
+#if defined(dragonfly)
+    struct xinpcb  *xig = NULL;
+#else
     struct xinpgen *xig = NULL;
+#endif
     netsnmp_inpcb  *nnew;
     int      StateMap[] = { 1, 2, 3, 4, 5, 8, 6, 10, 9, 7, 11 };
 
@@ -733,10 +746,19 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
      *  Unpick this into the constituent 'xinpgen' structures, and extract
      *     the 'inpcb' elements into a linked list (built in reverse)
      */
+#if defined(dragonfly)
+    xig = (struct xinpcb  *) tcpcb_buf;
+#else
     xig = (struct xinpgen *) tcpcb_buf;
     xig = (struct xinpgen *) ((char *) xig + xig->xig_len);
+#endif
 
-    while (xig && (xig->xig_len > sizeof(struct xinpgen))) {
+#if defined(dragonfly)
+    while (xig && ((char *)xig + xig->xi_len < tcpcb_buf + len))
+#else
+    while (xig && (xig->xig_len > sizeof(struct xinpgen)))
+#endif
+    {
         nnew = SNMP_MALLOC_TYPEDEF(netsnmp_inpcb);
         if (!nnew)
             break;
@@ -747,14 +769,22 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
         memcpy(&(nnew->pcb), &(((NS_ELEM *) xig)->xt_inp),
                            sizeof(struct inpcb));
 
-	nnew->inp_next = tcp_head;
-	tcp_head   = nnew;
+	if (nnew->pcb.inp_vflag & INP_IPV6)
+	    free(nnew);
+	else {
+	    nnew->inp_next = tcp_head;
+	    tcp_head   = nnew;
+	}
+#if defined(dragonfly)
+        xig = (struct xinpcb  *) ((char *) xig + xig->xi_len);
+#else
         xig = (struct xinpgen *) ((char *) xig + xig->xig_len);
+#endif
     }
 
     free(tcpcb_buf);
     if (tcp_head) {
-        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table\n"));
+        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table (sysctl)\n"));
         return 0;
     }
     DEBUGMSGTL(("mibII/tcpTable", "Failed to load TCP Table (sysctl)\n"));
@@ -762,7 +792,7 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
 }
 #undef NS_ELEM
 
-#else		/* (defined(CAN_USE_SYSCTL) && defined(TCPCTL_PCBLIST)) */
+#else		/* (defined(NETSNMP_CAN_USE_SYSCTL) && defined(TCPCTL_PCBLIST)) */
 #ifdef PCB_TABLE
 int
 tcpTable_load(netsnmp_cache *cache, void *vmagic)
@@ -789,8 +819,15 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
         nnew = SNMP_MALLOC_TYPEDEF(netsnmp_inpcb);
         if (!nnew)
             break;
-        klookup((unsigned long) entry, (char *)&(nnew->pcb), sizeof(struct inpcb));
-        klookup((int) nnew->pcb.inp_ppcb, (char *)&tcpcb, sizeof(struct tcpcb));
+        if (!NETSNMP_KLOOKUP(entry, (char *)&(nnew->pcb), sizeof(struct inpcb))) {
+            DEBUGMSGTL(("mibII/tcpTable:TcpTable_load", "klookup failed\n"));
+            break;
+        }
+
+        if (!NETSNMP_KLOOKUP(nnew->pcb.inp_ppcb, (char *)&tcpcb, sizeof(struct tcpcb))) {
+            DEBUGMSGTL(("mibII/tcpTable:TcpTable_load", "klookup failed\n"));
+            break;
+        }
 	nnew->state = StateMap[tcpcb.t_state];
         if (nnew->state == 5 /* established */ ||
             nnew->state == 8 /*  closeWait  */ )
@@ -805,7 +842,7 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
     }
 
     if (tcp_head) {
-        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table\n"));
+        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table (pcb_table)\n"));
         return 0;
     }
     DEBUGMSGTL(("mibII/tcpTable", "Failed to load TCP Table (pcb_table)\n"));
@@ -843,8 +880,14 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
         nnew = SNMP_MALLOC_TYPEDEF(netsnmp_inpcb);
         if (!nnew)
             break;
-        klookup((unsigned long) entry, (char *)&(nnew->pcb), sizeof(struct inpcb));
-        klookup((int) nnew->pcb.inp_ppcb, (char *)&tcpcb, sizeof(struct tcpcb));
+        if (!NETSNMP_KLOOKUP(entry, (char *)&(nnew->pcb), sizeof(struct inpcb))) {
+            DEBUGMSGTL(("mibII/tcpTable:tcpTable_load", "klookup failed\n"));
+            break;
+        }
+        if (!NETSNMP_KLOOKUP(nnew->pcb.inp_ppcb, (char *)&tcpcb, sizeof(struct tcpcb))) {
+            DEBUGMSGTL(("mibII/tcpTable:tcpTable_load", "klookup failed\n"));
+            break;
+        }
 	nnew->state    = StateMap[tcpcb.t_state];
         if (nnew->state == 5 /* established */ ||
             nnew->state == 8 /*  closeWait  */ )
@@ -859,7 +902,7 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
     }
 
     if (tcp_head) {
-        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table\n"));
+        DEBUGMSGTL(("mibII/tcpTable", "Loaded TCP Table (tcp_symbol)\n"));
         return 0;
     }
     DEBUGMSGTL(("mibII/tcpTable", "Failed to load TCP Table (tcp_symbol)\n"));
@@ -875,7 +918,7 @@ tcpTable_load(netsnmp_cache *cache, void *vmagic)
 }
 #endif				/* UDB_SYMBOL */
 #endif				/* PCB_TABLE */
-#endif		/* (defined(CAN_USE_SYSCTL) && defined(TCPCTL_PCBLIST)) */
+#endif		/* (defined(NETSNMP_CAN_USE_SYSCTL) && defined(TCPCTL_PCBLIST)) */
 #endif                          /* WIN32 cygwin */
 #endif                          /* linux */
 #endif                          /* solaris2 */
