@@ -16,6 +16,7 @@
 #include <net-snmp/agent/table.h>
 #include <net-snmp/agent/table_data.h>
 #include <net-snmp/agent/table_dataset.h>
+#include "mibgroup/mibII/sysORTable.h"
 #include "notification_log.h"
 
 /*
@@ -57,6 +58,8 @@ static u_long   max_age = 1440; /* 1440 = 24 hours, which is the mib default */
 static netsnmp_table_data_set *nlmLogTable;
 static netsnmp_table_data_set *nlmLogVarTable;
 
+static oid nlm_module_oid[] = { SNMP_OID_MIB2, 92 }; /* NOTIFICATION-LOG-MIB::notificationLogMIB */
+
 static void
 netsnmp_notif_log_remove_oldest(int count)
 {
@@ -217,15 +220,6 @@ initialize_table_nlmLogVariableTable(const char * context)
                 "adding index nlmLogVariableIndex of type ASN_UNSIGNED to table nlmLogVariableTable\n"));
     netsnmp_table_dataset_add_index(table_set, ASN_UNSIGNED);
 
-    /*
-     * adding column nlmLogVariableIndex of type ASN_UNSIGNED and access
-     * of NoAccess 
-     */
-    DEBUGMSGTL(("initialize_table_nlmLogVariableTable",
-                "adding column nlmLogVariableIndex (#1) of type ASN_UNSIGNED to table nlmLogVariableTable\n"));
-    netsnmp_table_set_add_default_row(table_set,
-                                      COLUMN_NLMLOGVARIABLEINDEX,
-                                      ASN_UNSIGNED, 0, NULL, 0);
     /*
      * adding column nlmLogVariableID of type ASN_OBJECT_ID and access of
      * ReadOnly 
@@ -557,6 +551,9 @@ init_notification_log(void)
                                NETSNMP_DS_APPLICATION_ID,
                                NETSNMP_DS_AGENT_NOTIF_LOG_MAX);
 #endif
+
+    REGISTER_SYSOR_ENTRY(nlm_module_oid, 
+        "The MIB module for logging SNMP Notifications.");
 }
 
 void
@@ -564,6 +561,10 @@ shutdown_notification_log(void)
 {
     max_logged = 0;
     check_log_size(0, NULL);
+    netsnmp_delete_table_data_set(nlmLogTable);
+    nlmLogTable = NULL;
+
+    unregister_sysORTable(nlm_module_oid, OID_LENGTH(nlm_module_oid));
 }
 
 void
@@ -585,6 +586,7 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
     u_long          vbcount = 0;
     u_long          tmpul;
     int             col;
+    netsnmp_pdu    *orig_pdu = pdu;
 
     if (!nlmLogVarTable
         || netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
@@ -651,13 +653,14 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
     netsnmp_set_row_column(row, COLUMN_NLMLOGCONTEXTNAME, ASN_OCTET_STR,
                            pdu->contextName, pdu->contextNameLen);
 
+    if (pdu->command == SNMP_MSG_TRAP)
+	pdu = convert_v1pdu_to_v2(orig_pdu);
     for (vptr = pdu->variables; vptr; vptr = vptr->next_variable) {
         if (snmp_oid_compare(snmptrapoid, snmptrapoid_len,
                              vptr->name, vptr->name_length) == 0) {
             netsnmp_set_row_column(row, COLUMN_NLMLOGNOTIFICATIONID,
                                    ASN_OBJECT_ID, vptr->val.string,
                                    vptr->val_len);
-
         } else {
             netsnmp_table_row *myrow;
             myrow = netsnmp_create_table_data_row();
@@ -749,6 +752,9 @@ log_notification(netsnmp_pdu *pdu, netsnmp_transport *transport)
         }
     }
 
+    if (pdu != orig_pdu)
+        snmp_free_pdu( pdu );
+
     /*
      * store the row 
      */
