@@ -77,6 +77,7 @@
 
 #ifdef USING_IF_MIB_IFTABLE_IFTABLE_MODULE
 #include "if-mib/ifTable/ifTable.h"
+#include "if-mib/ifTable/ifTable_defs.h"
 #else
 /*
  * This is used, because the TUNNEL-MIB augments ifTable. 
@@ -414,7 +415,7 @@ updateTunnels(void)
      */
     for (tunnel = tunnels; tunnel; tunnel = tunnel->next) {
         DEBUGMSG(("tunnel",
-                  "updateTunnels(): updating %s (index=%d)\n",
+                  "updateTunnels(): updating %s (index=%lu)\n",
                   tunnel->ifname, tunnel->ifindex));
         updateTunnel(tunnel);
     }
@@ -460,7 +461,7 @@ updateTunnels(void)
             last_tunnel = tunnel;
 
             DEBUGMSG(("tunnel",
-                      "updateTunnels(): added %s (index=%d state=%d)\n",
+                      "updateTunnels(): added %s (index=%lu state=%d)\n",
                       tunnel->ifname, tunnel->ifindex, tunnel->active));
         }
         if (type == 0)
@@ -482,7 +483,7 @@ getTunnelByIfIndex(int index)
             if (!tunnel->active)
                 break;
             DEBUGMSG(("tunnel",
-                      "%s (index=%d)\n", tunnel->ifname, tunnel->ifindex));
+                      "%s (index=%lu)\n", tunnel->ifname, tunnel->ifindex));
             return tunnel;
         }
     }
@@ -504,7 +505,7 @@ getNextTunnelByIfIndex(int index)
             if (!tunnel->active)
                 continue;
             DEBUGMSG(("tunnel",
-                      "%s (index=%d)\n", tunnel->ifname, tunnel->ifindex));
+                      "%s (index=%lu)\n", tunnel->ifname, tunnel->ifindex));
             return tunnel;
         }
     }
@@ -548,7 +549,7 @@ getTunnelByConfigOid(oid * name, size_t * length)
             if (!tunnel->active)
                 break;
             DEBUGMSG(("tunnel",
-                      "%s (index=%d)\n", tunnel->ifname, tunnel->ifindex));
+                      "%s (index=%lu)\n", tunnel->ifname, tunnel->ifindex));
             return tunnel;
         }
     }
@@ -592,7 +593,7 @@ getNextTunnelByConfigOid(oid * name, size_t * length)
 
     if (last_tunnel) {
         DEBUGMSG(("tunnel",
-                  "%s (index=%d)\n",
+                  "%s (index=%lu)\n",
                   last_tunnel->ifname, last_tunnel->ifindex));
     } else {
         DEBUGMSG(("tunnel", "NONE\n"));
