@@ -45,9 +45,11 @@
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
+#if !defined(dragonfly)
 #ifdef HAVE_SYS_VNODE_H
 #include <sys/vnode.h>
 #endif
+#endif
 #ifdef HAVE_UFS_UFS_QUOTA_H
 #include <ufs/ufs/quota.h>
 #endif
@@ -73,6 +75,9 @@
 #if HAVE_SYS_VFS_H
 #include <sys/vfs.h>
 #endif
+#if defined(__FreeBSD__) && __FreeBSD_version >= 700055 	/* Or HAVE_SYS_UCRED_H */
+#include <sys/ucred.h>
+#endif
 #if defined(HAVE_STATFS)
 #if HAVE_SYS_MOUNT_H
 #include <sys/mount.h>
@@ -163,9 +168,14 @@ struct diskpart {
     int             minpercent;
 };
 
-int             numdisks;
+#ifndef  INT32_MAX
+#define  INT32_MAX  0x7fffffff
+#endif
+
+
+unsigned int    numdisks;
 int             allDisksIncluded = 0;
-int             maxdisks = 0;
+unsigned int    maxdisks = 0;
 struct diskpart *disks;
 
 struct variable2 extensible_disk_variables[] = {
@@ -215,7 +225,7 @@ init_disk(void)
 static void
 disk_free_config(void)
 {
-  int             i;
+  unsigned int             i;
 
   numdisks = 0;
   for (i = 0; i < maxdisks; i++) {    /* init/erase disk db */
@@ -394,10 +404,10 @@ add_device(char *path, char *device, int minspace, int minpercent, int override)
   else if(index == -1){
     /* add if and only if the device was found */
     if(device[0] != 0) {
-      copy_nword(path, disks[numdisks].path, 
-		 sizeof(disks[numdisks].path));
-      copy_nword(device, disks[numdisks].device, 
-		 sizeof(disks[numdisks].device));
+      /* The following buffers are cleared above, no need to add '\0' */
+      strncpy(disks[numdisks].path, path, sizeof(disks[numdisks].path) - 1);
+      strncpy(disks[numdisks].device, device,
+              sizeof(disks[numdisks].device) - 1);
       disks[numdisks].minimumspace = minspace;
       disks[numdisks].minpercent   = minpercent;
       numdisks++;  
@@ -420,7 +430,7 @@ modify_disk_parameters(int index, int minspace, int minpercent)
 
 int disk_exists(char *path) 
 {
-  int index;
+  unsigned int index;
   for(index = 0; index < numdisks; index++) {
     DEBUGMSGTL(("ucd-snmp/disk", "Checking for %s. Found %s at %d\n", path, disks[index].path, index));
     if(strcmp(path, disks[index].path) == 0) {
@@ -460,7 +470,12 @@ find_and_add_allDisks(int minpercent)
 #if HAVE_GETMNTENT
 #if HAVE_SETMNTENT
   mntfp = setmntent(ETC_MNTTAB, "r");
-  while (NULL != (mntent = getmntent(mntfp))) {
+  if (!mntfp) {
+      snprintf( tmpbuf, sizeof(tmpbuf), "Can't open %s (setmntent)\n", ETC_MNTTAB );
+      config_perror(tmpbuf);
+      return;
+  }
+  while (mntfp && NULL != (mntent = getmntent(mntfp))) {
     add_device(mntent->mnt_dir, mntent->mnt_fsname, -1, minpercent, 0);
     dummy = 1;
   }
@@ -473,6 +488,11 @@ find_and_add_allDisks(int minpercent)
   }
 #else                           /* getmentent but not setmntent */
   mntfp = fopen(ETC_MNTTAB, "r");
+  if (!mntfp) {
+      snprintf( tmpbuf, sizeof(tmpbuf), "Can't open %s (fopen)\n", ETC_MNTTAB );
+      config_perror(tmpbuf);
+      return;
+  }
   while ((i = getmntent(mntfp, &mnttab)) == 0) {
     add_device(mnttab.mnt_mountp, mnttab.mnt_special, -1, minpercent, 0);
     dummy = 1;
@@ -491,6 +511,18 @@ find_and_add_allDisks(int minpercent)
     dummy = 1;
   }
   endfsent();			/* close /etc/fstab */
+#if defined(__FreeBSD__) && __FreeBSD_version >= 700055
+  {
+    struct statfs *mntbuf;
+    size_t i, mntsize;
+    mntsize = getmntinfo(&mntbuf, MNT_NOWAIT);
+    for (i = 0; i < mntsize; i++) {
+      if (strncmp(mntbuf[i].f_fstypename, "zfs", 3) == 0) {
+	add_device(mntbuf[i].f_mntonname, mntbuf[i].f_mntfromname, -1, minpercent, 0);
+      }
+    }
+  }
+#endif
   if(dummy != 0) {
     /*
      * dummy clause for else below
@@ -557,10 +589,15 @@ find_device(char *path)
 #if HAVE_GETMNTENT
 #if HAVE_SETMNTENT
   mntfp = setmntent(ETC_MNTTAB, "r");
-  while (NULL != (mntent = getmntent(mntfp)))
+  if (!mntfp) {
+      snprintf( tmpbuf, sizeof(tmpbuf), "Can't open %s (setmntent)\n", ETC_MNTTAB );
+      config_perror(tmpbuf);
+      return NULL;
+  }
+  while (mntfp && NULL != (mntent = getmntent(mntfp)))
     if (strcmp(path, mntent->mnt_dir) == 0) {
-      copy_nword(mntent->mnt_fsname, device,
-		 sizeof(device));
+      strncpy(device, mntent->mnt_fsname, sizeof(device));
+      device[sizeof(device) - 1] = '\0';
       DEBUGMSGTL(("ucd-snmp/disk", "Disk:  %s\n",
 		  mntent->mnt_fsname));
       break;
@@ -572,6 +609,11 @@ find_device(char *path)
     endmntent(mntfp);
 #else                           /* getmentent but not setmntent */
   mntfp = fopen(ETC_MNTTAB, "r");
+  if (!mntfp) {
+      snprintf( tmpbuf, sizeof(tmpbuf), "Can't open %s (fopen)\n", ETC_MNTTAB );
+      config_perror(tmpbuf);
+      return NULL;
+  }
   while ((i = getmntent(mntfp, &mnttab)) == 0)
     if (strcmp(path, mnttab.mnt_mountp) == 0)
       break;
@@ -581,16 +623,16 @@ find_device(char *path)
     }
   fclose(mntfp);
   if (i == 0) {
-    copy_nword(mnttab.mnt_special, device,
-	       sizeof(device));
+    strncpy(device, mnttab.mnt_special, sizeof(device));
+    device[sizeof(device) - 1] = '\0';
   }
 #endif /* HAVE_SETMNTENT */
 #elif HAVE_FSTAB_H
   stat(path, &stat1);
   setfsent();
   if ((fstab = getfsfile(path))) {
-    copy_nword(fstab->fs_spec, device,
-	       sizeof(device));
+    strncpy(device, fstab->fs_spec, sizeof(device));
+    device[sizeof(device) - 1] = '\0';
   }
   endfsent();
   if (device[0] != '\0') {
@@ -601,7 +643,8 @@ find_device(char *path)
 
 #elif HAVE_STATFS
   if (statfs(path, &statf) == 0) {
-    copy_nword(statf.f_mntfromname, device, sizeof(device));
+    strncpy(device, statf.f_mntfromname, sizeof(device) - 1);
+    device[sizeof(device) - 1] = '\0';
     DEBUGMSGTL(("ucd-snmp/disk", "Disk:  %s\n",
 		statf.f_mntfromname));
   }
@@ -637,7 +680,8 @@ var_extensible_disk(struct variable *vp,
                     size_t * var_len, WriteMethod ** write_method)
 {
 
-    int             percent, iserror, disknum = 0;
+    int             percent, iserror;
+    unsigned int    disknum = 0;
 #if !defined(HAVE_SYS_STATVFS_H) && !defined(HAVE_STATFS)
     double          totalblks, free, used, avail, availblks;
 #else
@@ -675,6 +719,8 @@ tryAgain:
         (vp, name, length, exact, var_len, write_method, numdisks))
         return (NULL);
     disknum = name[*length - 1] - 1;
+	if (disknum > maxdisks)
+		return NULL;
     switch (vp->magic) {
     case MIBINDEX:
         long_ret = disknum + 1;
@@ -730,7 +776,7 @@ tryAgain:
 #endif
     avail = (long)(vfs.f_bavail * multiplier);
     iserror = (disks[disknum].minimumspace >= 0 ?
-               avail < disks[disknum].minimumspace :
+               vfs.f_bavail < (disks[disknum].minimumspace/multiplier) :
                100 - percent <= disks[disknum].minpercent) ? 1 : 0;
 #if defined(STRUCT_STATVFS_HAS_F_FILES) || defined STRUCT_STATFS_HAS_F_FAVAIL
     percent_inode = vfs.f_favail <= 0 ? 100 :
@@ -746,12 +792,22 @@ tryAgain:
 #endif /* defined(STRUCT_STATVFS_HAS_F_FILES) */ 
     switch (vp->magic) {
     case DISKTOTAL:
-        long_ret = (long)(vfs.f_blocks * multiplier);
+        if ( vfs.f_blocks > ( INT32_MAX / multiplier ))
+            long_ret = INT32_MAX;
+        else
+            long_ret = (long)(vfs.f_blocks * multiplier);
         return ((u_char *) (&long_ret));
     case DISKAVAIL:
-        return ((u_char *) (&avail));
+        if ( vfs.f_bavail > ( INT32_MAX / multiplier ))
+            long_ret = INT32_MAX;
+        else
+            long_ret = (long)(vfs.f_bavail * multiplier);
+        return ((u_char *) (&long_ret));
     case DISKUSED:
-        long_ret = (long)((vfs.f_blocks - vfs.f_bfree) * multiplier);
+        if ( (vfs.f_blocks - vfs.f_bfree) > ( INT32_MAX / multiplier ))
+            long_ret = INT32_MAX;
+        else
+            long_ret = (long)((vfs.f_blocks - vfs.f_bfree) * multiplier);
         return ((u_char *) (&long_ret));
     case DISKPERCENT:
         long_ret = percent;
@@ -823,13 +879,22 @@ tryAgain:
             : 100 - percent <= disks[disknum].minpercent) ? 1 : 0;
     switch (vp->magic) {
     case DISKTOTAL:
-        long_ret = (long)(totalblks * multiplier);
+        if ( totalblks > ( INT32_MAX / multiplier ))
+            long_ret = INT32_MAX;
+        else
+            long_ret = (long)(totalblks * multiplier);
         return ((u_char *) (&long_ret));
     case DISKAVAIL:
-        long_ret = (long)(avail * multiplier);
+        if ( avail > ( INT32_MAX / multiplier ))
+            long_ret = INT32_MAX;
+        else
+            long_ret = (long)(avail * multiplier);
         return ((u_char *) (&long_ret));
     case DISKUSED:
-        long_ret = (long)(used * multiplier);
+        if ( used > (  INT32_MAX / multiplier ))
+            long_ret = INT32_MAX;
+        else
+            long_ret = (long)(used * multiplier);
         return ((u_char *) (&long_ret));
     case DISKPERCENT:
         long_ret = percent;
