@@ -72,7 +72,7 @@ static int ps_numdisks;			/* number of disks in system, may change while running
 #include <sys/param.h>
 #if __FreeBSD_version >= 500101
 #include <sys/resource.h>       /* for CPUSTATES in devstat.h */
-#else
+#elif !defined(dragonfly)
 #include <sys/dkstat.h>
 #endif
 #include <devstat.h>
@@ -321,7 +321,8 @@ getstats(void)
     time_t          now;
     int             mib[2];
     char           *t, *tp;
-    int             size, dkn_size, i;
+    size_t          size, dkn_size;
+    int             i;
 
     now = time(NULL);
     if (cache_time + CACHE_TIMEOUT > now) {
@@ -768,6 +769,7 @@ int getstats(void)
 		    &pTemp->running, &pTemp->use, &pTemp->aveq);
             if (rc != 15) {
                snmp_log(LOG_ERR, "diskio.c: cannot find statistics in /proc/partitions\n");
+               fclose(parts);
                return 1;
             }
 	    head.length++;
