@@ -360,8 +360,8 @@ get_proc_instance(struct myproc *proc, oid inst)
  * these are for keeping track of the proc array 
  */
 
-static int      nproc = 0;
-static int      onproc = -1;
+static size_t   nproc = 0;
+static size_t   onproc = -1;
 static struct kinfo_proc *pbase = 0;
 
 int
@@ -468,7 +468,7 @@ sh_count_procs(char *procname)
           continue;
       if (fgets(cmdline, sizeof(cmdline), status) == NULL) {
           fclose(status);
-          break;
+          continue;
       }
       /* Grab the state of the process as well
        * (so we can ignore zombie processes)
