@@ -1,7 +1,7 @@
 /*
  * object_monitor.c
  *
- * $Id: object_monitor.c,v 1.7 2006/01/25 16:25:52 dts12 Exp $
+ * $Id: object_monitor.c 19118 2010-06-28 00:21:46Z tanders $
  *
  * functions and data structures for cooperating code to monitor objects.
  *
@@ -25,7 +25,7 @@
 
 #if ! defined TRUE
 #  define TRUE 1
-#elsif TRUE != 1
+#elif TRUE != 1
 error "TRUE != 1"
 #endif
 /**************************************************************************
