@@ -275,7 +275,7 @@ usage(char *prog)
     printf("  -c FILE[,...]\t\tread FILE(s) as configuration file(s)\n");
     printf("  -C\t\t\tdo not read the default configuration files\n");
     printf("  -d\t\t\tdump sent and received SNMP packets\n");
-    printf("  -DTOKEN[,...]\tturn on debugging output for the given TOKEN(s)\n"
+    printf("  -D[TOKEN[,...]]\tturn on debugging output for the given TOKEN(s)\n"
 	   "\t\t\t  (try ALL for extremely verbose output)\n"
 	   "\t\t\t  Don't put space(s) between -D and TOKEN(s).\n");
     printf("  -f\t\t\tdo not fork from the shell\n");
@@ -1040,11 +1040,8 @@ main(int argc, char *argv[])
     SnmpTrapNodeDown();
     DEBUGMSGTL(("snmpd/main", "Bye...\n"));
     snmp_shutdown(app_name);
-#ifdef SHUTDOWN_AGENT_CLEANLY /* broken code */
-    /* these attempt to free all known memory, but result in double frees */
     shutdown_master_agent();
     shutdown_agent();
-#endif
 
     if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID, 
 				NETSNMP_DS_AGENT_LEAVE_PIDFILE) &&
@@ -1062,6 +1059,40 @@ main(int argc, char *argv[])
     return 0;
 }                               /* End main() -- snmpd */
 
+#if defined(WIN32)
+
+#include <process.h>
+#include <net-snmp/library/snmp_assert.h>
+
+static unsigned s_threadid;
+HANDLE s_thread_handle;
+
+static unsigned __stdcall wait_for_stdin(void* arg)
+{
+    if (getc(stdin) != EOF)
+        netsnmp_running = 0;
+    return 0;
+}
+
+static void create_stdin_waiter_thread(void)
+{
+    netsnmp_assert(s_thread_handle == 0);
+    s_thread_handle = (HANDLE)_beginthreadex(0, 0, wait_for_stdin, 0, 0, &s_threadid);
+    netsnmp_assert(s_thread_handle != 0);
+}
+
+static void join_stdin_waiter_thread(void)
+{
+    int result;
+
+    netsnmp_assert(s_thread_handle != 0);
+    result = WaitForSingleObject(s_thread_handle, 1000);
+    netsnmp_assert(result != WAIT_TIMEOUT);
+    CloseHandle(s_thread_handle);
+    s_thread_handle = 0;
+}
+#endif
+
 /*******************************************************************-o-******
  * receive
  *
@@ -1091,6 +1122,10 @@ receive(void)
      */
     reconfig = 0;
 
+#if defined(WIN32)
+    create_stdin_waiter_thread();
+#endif
+
     /*
      * Loop-forever: execute message handlers for sockets with data
      */
@@ -1232,6 +1267,10 @@ receive(void)
 
     }                           /* endwhile */
 
+#if defined(WIN32)
+    join_stdin_waiter_thread();
+#endif
+
     snmp_log(LOG_INFO, "Received TERM or STOP signal...  shutting down...\n");
     return 0;
 
