@@ -314,7 +314,7 @@ usage(void)
     fprintf(stderr,
             "  -C\t\t\tdo not read the default configuration files\n");
     fprintf(stderr, "  -d\t\t\tdump sent and received SNMP packets\n");
-    fprintf(stderr, "  -D\t\t\tturn on debugging output\n");
+    fprintf(stderr, "  -D[TOKEN[,...]]\t\tturn on debugging output\n");
     fprintf(stderr,
             "  -e\t\t\tprint event # (rising/falling alarm, etc.)\n");
     fprintf(stderr, "  -f\t\t\tdo not fork from the shell\n");
@@ -1477,7 +1477,9 @@ void
 trapd_update_config(void)
 {
     free_config();
+#ifdef USING_MIBII_VACM_CONF_MODULE
     vacm_standard_views(0,0,NULL,NULL);
+#endif
     read_configs();
 }
 
