@@ -53,10 +53,10 @@ extern          "C" {
             oid            *objid;
             u_char         *bitstring;
             struct counter64 *counter64;
-#ifdef OPAQUE_SPECIAL_TYPES
+#ifdef NETSNMP_WITH_OPAQUE_SPECIAL_TYPES
             float          *floatVal;
             double         *doubleVal;
-#endif                          /* OPAQUE_SPECIAL_TYPES */
+#endif                          /* NETSNMP_WITH_OPAQUE_SPECIAL_TYPES */
         } data;
         u_long          data_len;
 
@@ -77,24 +77,25 @@ extern          "C" {
 
     netsnmp_table_data_set *netsnmp_create_table_data_set(const char *);
     netsnmp_table_row *netsnmp_table_data_set_clone_row( netsnmp_table_row *row);
-    NETSNMP_INLINE void netsnmp_table_dataset_delete_all_data(
+    void netsnmp_table_dataset_delete_all_data(
                             netsnmp_table_data_set_storage *data);
-    NETSNMP_INLINE void netsnmp_table_dataset_delete_row(netsnmp_table_row *row);
+    void netsnmp_table_dataset_delete_row(netsnmp_table_row *row);
 
-    NETSNMP_INLINE void netsnmp_table_dataset_add_row(netsnmp_table_data_set
+    void netsnmp_table_dataset_add_row(netsnmp_table_data_set
                                                   *table,
                                                   netsnmp_table_row *row);
-    NETSNMP_INLINE void    
+    void
         netsnmp_table_dataset_replace_row(netsnmp_table_data_set *table,
                                           netsnmp_table_row *origrow,
                                           netsnmp_table_row *newrow);
-    NETSNMP_INLINE void netsnmp_table_dataset_remove_row(netsnmp_table_data_set
+    void netsnmp_table_dataset_remove_row(netsnmp_table_data_set
                                                      *table,
                                                      netsnmp_table_row *row);
-    NETSNMP_INLINE void    
+    void
         netsnmp_table_dataset_remove_and_delete_row(netsnmp_table_data_set
                                                     *table,
                                                     netsnmp_table_row *row);
+    void netsnmp_delete_table_data_set(netsnmp_table_data_set *table_set);
 
 /* ============================
  * DataSet API: Default row operations
@@ -130,7 +131,7 @@ extern          "C" {
     int netsnmp_register_table_data_set(netsnmp_handler_registration *,
                                         netsnmp_table_data_set *,
                                         netsnmp_table_registration_info *);
-    NETSNMP_INLINE netsnmp_table_data_set
+    netsnmp_table_data_set
         *netsnmp_extract_table_data_set(netsnmp_request_info *request);
     netsnmp_table_data_set_storage
         *netsnmp_extract_table_data_set_column(netsnmp_request_info *,
@@ -174,7 +175,7 @@ extern          "C" {
  * DataSet API: Index operations
  * ============================ */
 
-    NETSNMP_INLINE void netsnmp_table_dataset_add_index(netsnmp_table_data_set
+    void netsnmp_table_dataset_add_index(netsnmp_table_data_set
                                                     *table, u_char type);
 #if HAVE_STDARG_H
     void netsnmp_table_set_add_indexes(netsnmp_table_data_set *tset, ...);
