@@ -32,6 +32,15 @@
 
 #define UTMP_FILE _PATH_UTMP
 
+#define UDP_ADDRESSES_IN_HOST_ORDER 1
+
 #ifndef netbsd1
-# define netbsd1 1
+# define netbsd1 netbsd1
 #endif
+
+#if __NetBSD_Version__ >= 499005800
+#define NETBSD_STATS_VIA_SYSCTL
+#endif /* __NetBSD_Version__ >= 499005800 */
+
+/* define the extra mib modules that are supported */
+#define NETSNMP_INCLUDE_HOST_RESOURCES
