@@ -1029,7 +1029,29 @@ int *err_ind;
 {
    int status;
    long command = pdu->command;
+   char *tmp_err_str;
+
+   *err_num = 0;
+   *err_ind = 0;
    *response = NULL;
+   tmp_err_str = NULL;
+   memset(err_str, '\0', STR_BUF_SIZE);
+
+   if (ss == NULL) {
+       *err_num = 0;
+       *err_ind = SNMPERR_BAD_SESSION;
+       strncpy(err_str, snmp_api_errstring(*err_ind), STR_BUF_SIZE - 1);
+       goto done;
+   }
+
+   tmp_err_str = calloc(1, STR_BUF_SIZE);
+   if (tmp_err_str == NULL) {
+       *err_num = errno;
+       *err_ind = SNMPERR_MALLOC;
+       strncpy(err_str, snmp_api_errstring(*err_ind), STR_BUF_SIZE - 1);
+       goto done;
+   }
+
 retry:
 
    status = snmp_synch_response(ss, pdu, response);
@@ -1069,7 +1091,8 @@ retry:
             /* in SNMPv2c, SNMPv2u, SNMPv2*, and SNMPv3 PDUs */
             case SNMP_ERR_INCONSISTENTNAME:
             default:
-               strcat(err_str,(char*)snmp_errstring((*response)->errstat));
+               strncpy(err_str, (char*)snmp_errstring((*response)->errstat),
+		       STR_BUF_SIZE - 1);
                *err_num = (int)(*response)->errstat;
 	       *err_ind = (*response)->errindex;
                status = (*response)->errstat;
@@ -1079,8 +1102,9 @@ retry:
 
       case STAT_TIMEOUT:
       case STAT_ERROR:
-          strcat(err_str, (char*)snmp_api_errstring(ss->s_snmp_errno));
-          *err_num = ss->s_snmp_errno;
+	  snmp_sess_error(ss, err_num, err_ind, &tmp_err_str);
+	  strncpy(err_str, tmp_err_str, STR_BUF_SIZE - 1);
+	  err_str[STR_BUF_SIZE - 1] = '\0';
          break;
 
       default:
@@ -1088,6 +1112,10 @@ retry:
          *err_num = ss->s_snmp_errno;
          break;
    }
+done:
+   if (tmp_err_str) {
+   	free(tmp_err_str);
+   }
    if (_debug_level && *err_num) printf("XXX sync PDU: %s\n", err_str);
    return(status);
 }
@@ -1141,6 +1169,22 @@ py_netsnmp_attr_long(PyObject *obj, char * attr_name)
   return val;
 }
 
+static void *
+py_netsnmp_attr_void_ptr(PyObject *obj, char * attr_name)
+{
+  void *val = NULL;
+
+  if (obj && attr_name  && PyObject_HasAttrString(obj, attr_name)) {
+    PyObject *attr = PyObject_GetAttrString(obj, attr_name);
+    if (attr) {
+      val = PyLong_AsVoidPtr(attr);
+      Py_DECREF(attr);
+    }
+  }
+
+  return val;
+}
+
 static int
 py_netsnmp_verbose(void)
 {
@@ -1169,6 +1213,46 @@ py_netsnmp_attr_set_string(PyObject *obj, char *attr_name,
   return ret;
 }
 
+/**
+ * Update python session object error attributes.
+ *
+ * Copy the error info which may have been returned from __send_sync_pdu(...)
+ * into the python object. This will allow the python code to determine if
+ * an error occured during an snmp operation.
+ *
+ * Currently there are 3 attributes we care about
+ *
+ * ErrorNum - Copy of the value of netsnmp_session.s_errno. This is the system
+ * errno that was generated during our last call into the net-snmp library.
+ *
+ * ErrorInd - Copy of the value of netsmp_session.s_snmp_errno. These error
+ * numbers are separate from the system errno's and describe SNMP errors.
+ *
+ * ErrorStr - A string describing the ErrorInd that was returned during our last
+ * operation.
+ *
+ * @param[in] session The python object that represents our current Session
+ * @param[in|out] err_str A string describing err_ind
+ * @param[in|out] err_num The system errno currently stored by our session
+ * @param[in|out] err_ind The snmp errno currently stored by our session
+ */
+static void
+__py_netsnmp_update_session_errors(PyObject *session, char *err_str,
+                                    int err_num, int err_ind)
+{
+    PyObject *tmp_for_conversion; 
+
+    py_netsnmp_attr_set_string(session, "ErrorStr", err_str, STRLEN(err_str));
+
+    tmp_for_conversion = PyInt_FromLong(err_num);
+    PyObject_SetAttrString(session, "ErrorNum", tmp_for_conversion);
+    Py_DECREF(tmp_for_conversion);
+
+    tmp_for_conversion = PyInt_FromLong(err_ind);
+    PyObject_SetAttrString(session, "ErrorInd", tmp_for_conversion);
+    Py_DECREF(tmp_for_conversion);
+}
+
 static PyObject *
 netsnmp_create_session(PyObject *self, PyObject *args)
 {
@@ -1224,7 +1308,7 @@ netsnmp_create_session(PyObject *self, PyObject *args)
       printf("error:snmp_new_session: Couldn't open SNMP session");
   }
  end:
-  return Py_BuildValue("L", (long long)ss);
+  return PyLong_FromVoidPtr((void *)ss);
 }
 
 static PyObject *
@@ -1370,7 +1454,7 @@ netsnmp_create_session_v3(PyObject *self, PyObject *args)
   free (session.securityEngineID);
   free (session.contextEngineID);
 
-  return Py_BuildValue("i", (int)ss);
+  return PyLong_FromVoidPtr((void *)ss);
 }
 
 static PyObject *
@@ -1383,7 +1467,7 @@ netsnmp_delete_session(PyObject *self, PyObject *args)
     return NULL;
   }
 
-  ss = (SnmpSession *)py_netsnmp_attr_long(session, "sess_ptr");
+  ss = (SnmpSession *)py_netsnmp_attr_void_ptr(session, "sess_ptr");
 
   snmp_close(ss);
   return (Py_BuildValue(""));
@@ -1435,14 +1519,11 @@ netsnmp_get(PyObject *self, PyObject *args)
       goto done;
     }
 
-    ss = (SnmpSession *)py_netsnmp_attr_long(session, "sess_ptr");
+    ss = (SnmpSession *)py_netsnmp_attr_void_ptr(session, "sess_ptr");
 
     if (py_netsnmp_attr_string(session, "ErrorStr", &tmpstr, &tmplen) < 0) {
       goto done;
     }
-    memcpy(&err_str, tmpstr, tmplen);
-    err_num = py_netsnmp_attr_long(session, "ErrorNum");
-    err_ind = py_netsnmp_attr_long(session, "ErrorInd");
 
     if (py_netsnmp_attr_long(session, "UseLongNames"))
       getlabel_flag |= USE_LONG_NAMES;
@@ -1496,6 +1577,7 @@ netsnmp_get(PyObject *self, PyObject *args)
 
     status = __send_sync_pdu(ss, pdu, &response, retry_nosuch, 
 			     err_str, &err_num, &err_ind);
+    __py_netsnmp_update_session_errors(session, err_str, err_num, err_ind);
 
     /*
     ** Set up for numeric or full OID's, if necessary.  Save the old
@@ -1646,7 +1728,7 @@ netsnmp_getnext(PyObject *self, PyObject *args)
       goto done;
     }
 
-    ss = (SnmpSession *)py_netsnmp_attr_long(session, "sess_ptr");
+    ss = (SnmpSession *)py_netsnmp_attr_void_ptr(session, "sess_ptr");
 
     if (py_netsnmp_attr_string(session, "ErrorStr", &tmpstr, &tmplen) < 0) {
       goto done;
@@ -1711,6 +1793,7 @@ netsnmp_getnext(PyObject *self, PyObject *args)
 
     status = __send_sync_pdu(ss, pdu, &response, retry_nosuch, 
 			     err_str, &err_num, &err_ind);
+    __py_netsnmp_update_session_errors(session, err_str, err_num, err_ind);
 
     /*
     ** Set up for numeric or full OID's, if necessary.  Save the old
@@ -1869,7 +1952,7 @@ netsnmp_walk(PyObject *self, PyObject *args)
     if ((varbinds = PyObject_GetAttrString(varlist, "varbinds")) == NULL) {
       goto done;
     }
-    ss = (SnmpSession *)py_netsnmp_attr_long(session, "sess_ptr");
+    ss = (SnmpSession *)py_netsnmp_attr_void_ptr(session, "sess_ptr");
 
     if (py_netsnmp_attr_string(session, "ErrorStr", &tmpstr, &tmplen) < 0) {
       goto done;
@@ -1986,7 +2069,8 @@ netsnmp_walk(PyObject *self, PyObject *args)
 
       status = __send_sync_pdu(ss, pdu, &response, retry_nosuch, 
                                err_str, &err_num, &err_ind);
-      
+      __py_netsnmp_update_session_errors(session, err_str, err_num, err_ind);
+
       if (!response || !response->variables ||
           (response->variables->name_length < oid_arr_len) ||
           (memcmp(oid_arr, response->variables->name,
@@ -2143,7 +2227,7 @@ netsnmp_getbulk(PyObject *self, PyObject *args)
 
     if (varlist && (varbinds = PyObject_GetAttrString(varlist, "varbinds"))) {
       
-      ss = (SnmpSession *)py_netsnmp_attr_long(session, "sess_ptr");
+      ss = (SnmpSession *)py_netsnmp_attr_void_ptr(session, "sess_ptr");
 
       if (py_netsnmp_attr_string(session, "ErrorStr", &tmpstr, &tmplen) < 0) {
         goto done;
@@ -2204,6 +2288,7 @@ netsnmp_getbulk(PyObject *self, PyObject *args)
 
       status = __send_sync_pdu(ss, pdu, &response, retry_nosuch, 
 			       err_str, &err_num, &err_ind);
+      __py_netsnmp_update_session_errors(session, err_str, err_num, err_ind);
 
       /*
       ** Set up for numeric or full OID's, if necessary.  Save the old
@@ -2371,15 +2456,12 @@ netsnmp_set(PyObject *self, PyObject *args)
       goto done;
     }
 
-    ss = (SnmpSession *)py_netsnmp_attr_long(session, "sess_ptr");
+    ss = (SnmpSession *)py_netsnmp_attr_void_ptr(session, "sess_ptr");
 
     /* PyObject_SetAttrString(); */
     if (py_netsnmp_attr_string(session, "ErrorStr", &tmpstr, &tmplen) < 0) {
       goto done;
     }
-    memcpy(&err_str, tmpstr, tmplen);
-    err_num = py_netsnmp_attr_long(session, "ErrorNum");
-    err_ind = py_netsnmp_attr_long(session, "ErrorInd");
 
     use_enums = py_netsnmp_attr_long(session, "UseEnums");
 	
@@ -2458,13 +2540,14 @@ netsnmp_set(PyObject *self, PyObject *args)
 
     status = __send_sync_pdu(ss, pdu, &response, NO_RETRY_NOSUCH, 
 			     err_str, &err_num, &err_ind);
+    __py_netsnmp_update_session_errors(session, err_str, err_num, err_ind);
 
     if (response) snmp_free_pdu(response);
 
     if (status == STAT_SUCCESS)
       ret = Py_BuildValue("i",1); /* success, return True */
     else
-      ret = Py_BuildValue("i",0); /* success, return False */
+      ret = Py_BuildValue("i",0); /* fail, return False */
   } 
  done:
   SAFE_FREE(oid_arr);
