@@ -1,7 +1,7 @@
 /*
  * netsnmp_data_list.c
  *
- * $Id: data_list.c 16758 2007-12-19 22:39:31Z magfr $
+ * $Id: data_list.c 19639 2010-11-07 21:14:21Z magfr $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -68,6 +68,10 @@ netsnmp_create_data_list(const char *name, void *data,
     if (!node)
         return NULL;
     node->name = strdup(name);
+    if (!node->name) {
+      free(node);
+      return NULL;
+    }
     node->data = data;
     node->free_func = beer;
     return node;
