@@ -58,6 +58,10 @@ const struct in6_addr in6addr_any = IN6ADDR_ANY_INIT;
 #define SS_FAMILY __ss_family
 #endif
 
+#if defined(darwin)
+#include <stdint.h> /* for uint8_t */
+#endif
+
 #include <net-snmp/types.h>
 #include <net-snmp/output_api.h>
 #include <net-snmp/config_api.h>
@@ -1361,8 +1365,9 @@ netsnmp_udp6_ctor(void)
 {
     udp6Domain.name = netsnmp_UDPIPv6Domain;
     udp6Domain.name_length = sizeof(netsnmp_UDPIPv6Domain) / sizeof(oid);
+    udp6Domain.f_create_from_tstring     = NULL;
     udp6Domain.f_create_from_tstring_new = netsnmp_udp6_create_tstring;
-    udp6Domain.f_create_from_ostring = netsnmp_udp6_create_ostring;
+    udp6Domain.f_create_from_ostring     = netsnmp_udp6_create_ostring;
     udp6Domain.prefix = (const char**)calloc(5, sizeof(char *));
     udp6Domain.prefix[0] = "udp6";
     udp6Domain.prefix[1] = "ipv6";
