@@ -134,7 +134,11 @@ netsnmp_unix_recv(netsnmp_transport *t, void *buf, int size,
             return -1;
         };
         while (rc < 0) {
-            rc = recvfrom(t->sock, buf, size, 0, NULL, 0);
+#ifdef MSG_DONTWAIT
+            rc = recvfrom(t->sock, buf, size, MSG_DONTWAIT, NULL, NULL);
+#else
+            rc = recvfrom(t->sock, buf, size, 0, NULL, NULL);
+#endif
             if (rc < 0 && errno != EINTR) {
                 DEBUGMSGTL(("netsnmp_unix", "recv fd %d err %d (\"%s\")\n",
                             t->sock, errno, strerror(errno)));
@@ -459,8 +463,9 @@ netsnmp_unix_ctor(void)
     unixDomain.prefix = (const char**)calloc(2, sizeof(char *));
     unixDomain.prefix[0] = "unix";
 
+    unixDomain.f_create_from_tstring     = NULL;
     unixDomain.f_create_from_tstring_new = netsnmp_unix_create_tstring;
-    unixDomain.f_create_from_ostring = netsnmp_unix_create_ostring;
+    unixDomain.f_create_from_ostring     = netsnmp_unix_create_ostring;
 
     netsnmp_tdomain_register(&unixDomain);
 }
