@@ -22,6 +22,9 @@
 #include <winsock.h>
 #endif
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #if HAVE_DMALLOC_H
 #include <dmalloc.h>
 #endif
@@ -62,14 +65,6 @@ static int      debugindent = 0;
 static char     debugindentchars[] =
     "                                                                                ";
 
-/*
- * Prototype definitions 
- */
-void            debug_config_register_tokens(const char *configtoken,
-                                             char *tokens);
-void            debug_config_turn_on_debugging(const char *configtoken,
-                                               char *line);
-
 char           *
 debug_indent(void)
 {
