@@ -32,6 +32,9 @@ unsigned int    current_maj_num;
 unsigned int    current_min_num;
 struct snmp_enum_list_str *sliststorage;
 
+static void
+free_enum_list(struct snmp_enum_list *list);
+
 int
 init_snmp_enum(const char *type)
 {
@@ -120,7 +123,7 @@ se_read_conf(const char *word, char *cptr)
             cp2 = e_enum;
             while (*(cp2++) != ':')
                 ;
-            se_add_pair(major, minor, cp2, value);
+            se_add_pair(major, minor, strdup(cp2), value);
             if (!cp)
                 break;
         }
@@ -136,7 +139,7 @@ se_read_conf(const char *word, char *cptr)
             cp2 = e_enum;
             while (*(cp2++) != ':')
                 ;
-            se_add_pair_to_slist(e_name, cp2, value);
+            se_add_pair_to_slist(e_name, strdup(cp2), value);
             if (!cp)
                 break;
         }
@@ -200,7 +203,7 @@ se_find_list(unsigned int major, unsigned int minor)
 }
 
 int
-se_find_value_in_list(struct snmp_enum_list *list, char *label)
+se_find_value_in_list(struct snmp_enum_list *list, const char *label)
 {
     if (!list)
         return SE_DNE;          /* XXX: um, no good solution here */
@@ -228,7 +231,7 @@ se_find_free_value_in_list(struct snmp_enum_list *list)
 }
 
 int
-se_find_value(unsigned int major, unsigned int minor, char *label)
+se_find_value(unsigned int major, unsigned int minor, const char *label)
 {
     return se_find_value_in_list(se_find_list(major, minor), label);
 }
@@ -267,8 +270,10 @@ se_add_pair_to_list(struct snmp_enum_list **list, char *label, int value)
         return SE_DNE;
 
     while (*list) {
-        if ((*list)->value == value)
+        if ((*list)->value == value) {
+            free(label);
             return (SE_ALREADY_THERE);
+        }
         lastnode = (*list);
         (*list) = (*list)->next;
     }
@@ -302,8 +307,8 @@ se_add_pair(unsigned int major, unsigned int minor, char *label, int value)
 /*
  * remember a list of enums based on a lookup name.
  */
-struct snmp_enum_list *
-se_find_slist(const char *listname)
+static struct snmp_enum_list **
+se_find_slist_ptr(const char *listname)
 {
     struct snmp_enum_list_str *sptr, *lastp = NULL;
     if (!listname)
@@ -312,11 +317,17 @@ se_find_slist(const char *listname)
     for (sptr = sliststorage;
          sptr != NULL; lastp = sptr, sptr = sptr->next)
         if (sptr->name && strcmp(sptr->name, listname) == 0)
-            return sptr->list;
+            return &sptr->list;
 
     return NULL;
 }
 
+struct snmp_enum_list *
+se_find_slist(const char *listname)
+{
+    struct snmp_enum_list **ptr = se_find_slist_ptr(listname);
+    return ptr ? *ptr : NULL;
+}
 
 char           *
 se_find_label_in_slist(const char *listname, int value)
@@ -326,7 +337,7 @@ se_find_label_in_slist(const char *listname, int value)
 
 
 int
-se_find_value_in_slist(const char *listname, char *label)
+se_find_value_in_slist(const char *listname, const char *label)
 {
     return (se_find_value_in_list(se_find_slist(listname), label));
 }
@@ -347,8 +358,10 @@ se_add_pair_to_slist(const char *listname, char *label, int value)
     if (!created) {
         struct snmp_enum_list_str *sptr =
             SNMP_MALLOC_STRUCT(snmp_enum_list_str);
-        if (!sptr)
+        if (!sptr) {
+            free_enum_list(list);
             return SE_NOMEM;
+        }
         sptr->next = sliststorage;
         sptr->name = strdup(listname);
         sptr->list = list;
@@ -357,22 +370,28 @@ se_add_pair_to_slist(const char *listname, char *label, int value)
     return ret;
 }
 
+static void
+free_enum_list(struct snmp_enum_list *list)
+{
+    struct snmp_enum_list *next;
+
+    while (list) {
+        next = list->next;
+        SNMP_FREE(list->label);
+        SNMP_FREE(list);
+        list = next;
+    }
+}
+
 void
 clear_snmp_enum(void)
 {
     struct snmp_enum_list_str *sptr = sliststorage, *next = NULL;
-    struct snmp_enum_list *list = NULL, *nextlist = NULL;
-    int i;
+    int i, j;
 
     while (sptr != NULL) {
 	next = sptr->next;
-	list = sptr->list;
-	while (list != NULL) {
-	    nextlist = list->next;
-	    SNMP_FREE(list->label);
-	    SNMP_FREE(list);
-	    list = nextlist;
-	}
+	free_enum_list(sptr->list);
 	SNMP_FREE(sptr->name);
 	SNMP_FREE(sptr);
 	sptr = next;
@@ -381,8 +400,13 @@ clear_snmp_enum(void)
 
     if (snmp_enum_lists) {
         for (i = 0; i < SE_MAX_IDS; i++) {
-            if (snmp_enum_lists[i])
+            if (snmp_enum_lists[i]) {
+                for (j = 0; j < SE_MAX_SUBIDS; j++) {
+                    if (snmp_enum_lists[i][j])
+                        free_enum_list(snmp_enum_lists[i][j]);
+                }
                 SNMP_FREE(snmp_enum_lists[i]);
+            }
         }
         SNMP_FREE(snmp_enum_lists);
     }
@@ -410,8 +434,7 @@ se_clear_list(struct snmp_enum_list **list)
 void
 se_clear_slist(const char *listname)
 {
-    struct snmp_enum_list *list = se_find_slist(listname);
-    se_clear_list(&list);
+    se_clear_list(se_find_slist_ptr(listname));
 }
 
 void
@@ -444,17 +467,19 @@ se_clear_all_lists(void)
 main()
 {
     init_snmp_enum();
-    se_add_pair(1, 1, "hi", 1);
-    se_add_pair(1, 1, "there", 2);
+    se_add_pair(1, 1, strdup("hi"), 1);
+    se_add_pair(1, 1, strdup("there"), 2);
     printf("hi: %d\n", se_find_value(1, 1, "hi"));
     printf("2: %s\n", se_find_label(1, 1, 2));
 
     se_add_pair_to_slist("testing", "life, and everything", 42);
-    se_add_pair_to_slist("testing", "resturant at the end of the universe",
+    se_add_pair_to_slist("testing", "restaurant at the end of the universe",
                          2);
 
     printf("life, and everything: %d\n",
            se_find_value_in_slist("testing", "life, and everything"));
     printf("2: %s\n", se_find_label_in_slist("testing", 2));
+
+    se_clear_all_lists();
 }
 #endif                          /* TESTING */
