@@ -47,6 +47,9 @@
 #include <windows.h>
 #endif
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #if HAVE_DMALLOC_H
 #include <dmalloc.h>
 #endif
@@ -950,6 +953,9 @@ char *netsnmp_getenv(const char *name)
   /* Next try HKCU */
   if (temp == NULL)
   {
+    if (getenv("SNMP_IGNORE_WINDOWS_REGISTRY"))
+      return NULL;
+
     if (RegOpenKeyExA(
           HKEY_CURRENT_USER, 
           "SOFTWARE\\Net-SNMP", 
