@@ -199,10 +199,10 @@ vacm_parse_config_view(const char *token, char *line)
     vptr->viewStatus = view.viewStatus;
     vptr->viewStorageType = view.viewStorageType;
     vptr->viewType = view.viewType;
-    viewMask = (u_char *) vptr->viewMask;
+    viewMask = vptr->viewMask;
+    vptr->viewMaskLen = sizeof(vptr->viewMask);
     line =
-        read_config_read_octet_string(line, (u_char **) & viewMask,
-                                      &vptr->viewMaskLen);
+        read_config_read_octet_string(line, &viewMask, &vptr->viewMaskLen);
 }
 
 /*
@@ -456,7 +456,7 @@ netsnmp_view_get(struct vacm_viewEntry *head, const char *viewName,
             for (oidpos = 0;
                  found && oidpos < (int) vp->viewSubtreeLen - 1;
                  oidpos++) {
-                if (mode==VACM_MODE_IGNORE_MASK || (VIEW_MASK(vp, maskpos, mask)) != 0) {
+                if (mode==VACM_MODE_IGNORE_MASK || (VIEW_MASK(vp, maskpos, mask) != 0)) {
                     if (viewSubtree[oidpos] !=
                         vp->viewSubtree[oidpos + 1])
                         found = 0;
