@@ -31,6 +31,10 @@ labelFIN: \
 
 #endif /* mingw32 */
 
+
+#define CountOf(arr) ( sizeof(arr) / sizeof(arr[0]) )
+
+
     /*
      * External global variables used here
      */
@@ -40,7 +44,7 @@ labelFIN: \
      * This should be declared by the application, which wants to register as
      * windows service
      */
-extern LPTSTR app_name;
+extern LPTSTR app_name_long;
 
     /*
      * Declare global variable
@@ -105,7 +109,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
   SC_HANDLE hSCManager = NULL;
   SC_HANDLE hService = NULL;
   TCHAR szRegAppLogKey[] =
-    "SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\";
+    _T("SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\");
   TCHAR szRegKey[512];
   HKEY hKey = NULL;		/* Key to registry entry */
   HKEY hParamKey = NULL;	/* To store startup parameters */
@@ -128,12 +132,12 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
       }
 
     /*
-     * Generate the Command to be executed by SCM 
+     * Generate the command to be executed by the SCM 
      */
-    _snprintf (szServiceCommand, sizeof(szServiceCommand), "%s %s", szServicePath, _T ("-service"));
+    _sntprintf (szServiceCommand, CountOf(szServiceCommand), _T("%s %s"), szServicePath, _T ("-service"));
 
     /*
-     * Create the Desired service 
+     * Create the desired service 
      */
     hService = CreateService (hSCManager, lpszServiceName, lpszServiceDisplayName,
 			SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS,
@@ -145,7 +149,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 			      NULL);	/* password */
     if (hService == NULL)
       {
-	_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+	_sntprintf (MsgErrorString, CountOf(MsgErrorString), _T("%s %s"),
 		   _T ("Can't create service"), lpszServiceDisplayName);
         ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
 
@@ -154,7 +158,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
       }
 
     /*
-     * Create registry entries for EventLog 
+     * Create registry entries for the event log 
      */
     /*
      * Create registry Application event log key 
@@ -167,7 +171,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
      */
     if (RegCreateKey (HKEY_LOCAL_MACHINE, szRegKey, &hKey) != ERROR_SUCCESS)
       {
-	_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+	_sntprintf (MsgErrorString, CountOf(MsgErrorString), _T("%s %s"),
 		   _T ("is unable to create registry entries"), lpszServiceDisplayName);
         ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
         exitStatus = SERVICE_ERROR_CREATE_REGISTRY_ENTRIES;
@@ -177,7 +181,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
     /*
      * Add Event ID message file name to the 'EventMessageFile' subkey 
      */
-    RegSetValueEx (hKey, "EventMessageFile", 0, REG_EXPAND_SZ,
+    RegSetValueEx (hKey, _T("EventMessageFile"), 0, REG_EXPAND_SZ,
 		   (CONST BYTE *) szServicePath,
 		   _tcslen (szServicePath) + sizeof (TCHAR));
 
@@ -185,7 +189,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
      * Set the supported types flags. 
      */
     dwData = EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | EVENTLOG_INFORMATION_TYPE;
-    RegSetValueEx (hKey, "TypesSupported", 0, REG_DWORD,
+    RegSetValueEx (hKey, _T("TypesSupported"), 0, REG_DWORD,
 		   (CONST BYTE *) & dwData, sizeof (DWORD));
 
     /*
@@ -202,7 +206,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 	 * Create Registry Key path 
 	 */
 	_tcscpy (szRegKey, _T ("SYSTEM\\CurrentControlSet\\Services\\"));
-	_tcscat (szRegKey, app_name);
+	_tcscat (szRegKey, app_name_long);
 	hKey = NULL;
 
 	/*
@@ -211,7 +215,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 	if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_WRITE,
 			  &hKey) != ERROR_SUCCESS)
 	  {
-	    _snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+	    _sntprintf (MsgErrorString, CountOf(MsgErrorString), _T("%s %s"),
 		       _T ("is unable to create registry entries"),
 		       lpszServiceDisplayName);
             ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
@@ -224,12 +228,12 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 	 */
 	if (lpszServiceDescription != NULL)
 	  {
-	    if (RegSetValueEx (hKey, "Description", 0, REG_SZ,
+	    if (RegSetValueEx (hKey, _T("Description"), 0, REG_SZ,
 			       (CONST BYTE *) lpszServiceDescription,
 			       _tcslen (lpszServiceDescription) +
 			       sizeof (TCHAR)) != ERROR_SUCCESS)
 	      {
-		_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+		_sntprintf (MsgErrorString, CountOf(MsgErrorString), _T("%s %s"),
 			   _T ("is unable to create registry entries"),
 			   lpszServiceDisplayName);
                 ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
@@ -247,11 +251,11 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 	     * Create Subkey parameters 
 	     */
 	    if (RegCreateKeyEx
-		(hKey, "Parameters", 0, NULL,
+		(hKey, _T("Parameters"), 0, NULL,
 		 REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL,
 		 &hParamKey, NULL) != ERROR_SUCCESS)
 	      {
-		_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+		_sntprintf (MsgErrorString, CountOf(MsgErrorString), _T("%s %s"),
 			   _T ("is unable to create registry entries"),
 			   lpszServiceDisplayName);
                 ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
@@ -273,7 +277,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 
 	    for (j = 1; i < StartUpArg->Argc; i++, j++)
 	      {
-		_snprintf (szRegKey, sizeof(szRegKey), "%s%d", _T ("Param"), j);
+		_sntprintf (szRegKey, CountOf(szRegKey), _T("%s%d"), _T ("Param"), j);
 
 		/*
 		 * Create registry key 
@@ -284,7 +288,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
 		     _tcslen (StartUpArg->Argv[i]) +
 		     sizeof (TCHAR)) != ERROR_SUCCESS)
 		  {
-		    _snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s",
+		    _sntprintf (MsgErrorString, CountOf(MsgErrorString), _T("%s %s"),
 			       _T ("is unable to create registry entries"),
 			       lpszServiceDisplayName);
                     ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
@@ -302,13 +306,13 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
       }
 
     /*
-     * Ready to Log messages 
+     * Ready to log messages 
      */
 
     /*
      * Successfully registered as service 
      */
-    _snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s", lpszServiceName,
+    _sntprintf (MsgErrorString, CountOf(MsgErrorString), _T("%s %s"), lpszServiceName,
 	       _T ("successfully registered as a service"));
 
     /*
@@ -336,14 +340,14 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
      * Input - ServiceName
      */
 int
-UnregisterService (LPCSTR lpszServiceName, int quiet)
+UnregisterService (LPCTSTR lpszServiceName, int quiet)
 {
   TCHAR MsgErrorString[MAX_STR_SIZE];	/* Message or Error string */
   SC_HANDLE hSCManager = NULL;	/* SCM handle */
   SC_HANDLE hService = NULL;	/* Service Handle */
   SERVICE_STATUS sStatus;
   TCHAR szRegAppLogKey[] =
-    "SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\";
+    _T("SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\");
   TCHAR szRegKey[512];
   int exitStatus = 0;
 /*  HKEY hKey = NULL;		?* Key to registry entry */
@@ -366,7 +370,7 @@ UnregisterService (LPCSTR lpszServiceName, int quiet)
     hService = OpenService (hSCManager, lpszServiceName, SERVICE_ALL_ACCESS);
     if (hService == NULL)
       {
-	_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s", _T ("Can't open service"),
+	_sntprintf (MsgErrorString, CountOf(MsgErrorString), _T("%s %s"), _T ("Can't open service"),
 		   lpszServiceName);
         ProcessError (EVENTLOG_ERROR_TYPE, MsgErrorString, 1, quiet);
         exitStatus = SERVICE_ERROR_OPEN_SERVICE;       
@@ -391,7 +395,7 @@ UnregisterService (LPCSTR lpszServiceName, int quiet)
      */
     if (DeleteService (hService) == FALSE)
       {
-	_snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s", _T ("Can't delete service"),
+	_sntprintf (MsgErrorString, CountOf(MsgErrorString), _T("%s %s"), _T ("Can't delete service"),
 		   lpszServiceName);
 
 	/*
@@ -404,7 +408,7 @@ UnregisterService (LPCSTR lpszServiceName, int quiet)
     /*
      * Log "Service deleted successfully " message to eventlog
      */
-    _snprintf (MsgErrorString, sizeof(MsgErrorString), "%s %s", lpszServiceName, _T ("service deleted"));
+    _sntprintf (MsgErrorString, CountOf(MsgErrorString), _T("%s %s"), lpszServiceName, _T ("service deleted"));
     ProcessError (EVENTLOG_INFORMATION_TYPE, MsgErrorString, 0, quiet);
 
     /*
@@ -429,8 +433,7 @@ UnregisterService (LPCSTR lpszServiceName, int quiet)
 }
 
     /*
-     * To write message to Windows Event log
-     * Input - Event Type, Message string
+     * Write a message to the Windows event log.
      */
 VOID
 WriteToEventLog (WORD wType, LPCTSTR pszFormat, ...)
@@ -439,15 +442,16 @@ WriteToEventLog (WORD wType, LPCTSTR pszFormat, ...)
   LPTSTR LogStr[1];
   va_list ArgList;
   HANDLE hEventSource = NULL;
+
   va_start (ArgList, pszFormat);
-  _vsnprintf (szMessage, sizeof(szMessage), pszFormat, ArgList);
+  _vsntprintf (szMessage, CountOf(szMessage), pszFormat, ArgList);
   va_end (ArgList);
   LogStr[0] = szMessage;
-  hEventSource = RegisterEventSource (NULL, app_name);
+  hEventSource = RegisterEventSource (NULL, app_name_long);
   if (hEventSource == NULL)
     return;
   ReportEvent (hEventSource, wType, 0,
-	       DISPLAY_MSG,	/* To Just output the text to event log */
+	       DISPLAY_MSG,
 	       NULL, 1, 0, LogStr, NULL);
   DeregisterEventSource (hEventSource);
 }
@@ -465,7 +469,7 @@ WriteToEventLog (WORD wType, LPCTSTR pszFormat, ...)
 INT
 ParseCmdLineForServiceOption (int argc, TCHAR * argv[], int *quiet)
 {
-  int nReturn = RUN_AS_CONSOLE;	/* Defualted to run as console */
+  int nReturn = RUN_AS_CONSOLE;	/* default is to run as a console application */
 
   if (argc >= 2)
     {
@@ -504,7 +508,7 @@ ParseCmdLineForServiceOption (int argc, TCHAR * argv[], int *quiet)
 }
 
     /*
-     * Write error message to Event Log, console or pop-up window
+     * Write error message to event log, console or pop-up window.
      *
      * If useGetLastError is 1, the last error returned from GetLastError()
      * is appended to pszMessage, separated by a ": ".
@@ -533,24 +537,24 @@ ProcessError (WORD eventLogType, LPCTSTR pszMessage, int useGetLastError, int qu
 		 MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
         (LPTSTR) & pErrorMsgTemp, 0, NULL);
 
-    _snprintf (pszMessageFull, sizeof(pszMessageFull), "%s: %s", pszMessage, pErrorMsgTemp);
+    _sntprintf (pszMessageFull, CountOf(pszMessageFull), _T("%s: %s"), pszMessage, pErrorMsgTemp);
     if (pErrorMsgTemp) {
       LocalFree (pErrorMsgTemp);
       pErrorMsgTemp = NULL;
     }
   }
   else {
-    _snprintf (pszMessageFull, sizeof(pszMessageFull), "%s", pszMessage);
+    _sntprintf (pszMessageFull, CountOf(pszMessageFull), _T("%s"), pszMessage);
   }
   
-  hEventSource = RegisterEventSource (NULL, app_name);
+  hEventSource = RegisterEventSource (NULL, app_name_long);
   if (hEventSource != NULL) {
     pErrorMsgTemp = pszMessageFull;
     
     if (ReportEvent (hEventSource, 
           eventLogType, 
           0,
-          DISPLAY_MSG,	/* To Just output the text to event log */
+          DISPLAY_MSG,	/* just output the text to the event log */
           NULL, 
           1, 
           0, 
@@ -562,8 +566,7 @@ ProcessError (WORD eventLogType, LPCTSTR pszMessage, int useGetLastError, int qu
           FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (),
           MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
           (LPTSTR) & pErrorMsgTemp, 0, NULL);
-      
-      fprintf(stderr,"Could NOT lot to Event Log.  Error returned from ReportEvent(): %s\n",pErrorMsgTemp);
+      _ftprintf(stderr,_T("Could NOT lot to Event Log.  Error returned from ReportEvent(): %s\n"),pErrorMsgTemp);
       if (pErrorMsgTemp) {
         LocalFree (pErrorMsgTemp);
         pErrorMsgTemp = NULL;
@@ -573,21 +576,21 @@ ProcessError (WORD eventLogType, LPCTSTR pszMessage, int useGetLastError, int qu
     }
 
       if (quiet) {
-    fprintf(stderr,"%s\n",pszMessageFull);
+    _ftprintf(stderr,_T("%s\n"),pszMessageFull);
       }
       else {
     switch (eventLogType) {
       case EVENTLOG_INFORMATION_TYPE:
-        MessageBox (NULL, pszMessageFull, app_name, MB_ICONASTERISK);
+        MessageBox (NULL, pszMessageFull, app_name_long, MB_ICONASTERISK);
         break;
       case EVENTLOG_WARNING_TYPE:
-        MessageBox (NULL, pszMessageFull, app_name, MB_ICONEXCLAMATION);
+        MessageBox (NULL, pszMessageFull, app_name_long, MB_ICONEXCLAMATION);
         break;
       case EVENTLOG_ERROR_TYPE:
-        MessageBox (NULL, pszMessageFull, app_name, MB_ICONSTOP);
+        MessageBox (NULL, pszMessageFull, app_name_long, MB_ICONSTOP);
         break;
       default:
-        MessageBox (NULL, pszMessageFull, app_name, EVENTLOG_WARNING_TYPE);
+        MessageBox (NULL, pszMessageFull, app_name_long, EVENTLOG_WARNING_TYPE);
         break;
       }
     }
@@ -596,9 +599,9 @@ ProcessError (WORD eventLogType, LPCTSTR pszMessage, int useGetLastError, int qu
 }
 
     /*
-     *  To update current service status 
-     *  Sends the current service status to the SCM. Also updates
-     *  the global service status structure.
+     * Update current service status.
+     * Sends the current service status to the SCM. Also updates
+     * the global service status structure.
      */
 static BOOL
 UpdateServiceStatus (DWORD dwStatus, DWORD dwErrorCode, DWORD dwWaitHint)
@@ -636,7 +639,7 @@ UpdateServiceStatus (DWORD dwStatus, DWORD dwErrorCode, DWORD dwWaitHint)
 }
 
     /*
-     * Reports current Service status to SCM
+     * Reports current service status to SCM
      */
 static BOOL
 ReportCurrentServiceStatus ()
@@ -645,23 +648,21 @@ ReportCurrentServiceStatus ()
 }
 
     /*
-     * The ServiceMain function to start service.
+     * ServiceMain function.
      */
 VOID WINAPI
 ServiceMain (DWORD argc, LPTSTR argv[])
 {
   SECURITY_ATTRIBUTES SecurityAttributes;
-  DWORD dwThreadId;
+  unsigned threadId;
 
   /*
-   * Input Arguments to function startup 
+   * Input arguments
    */
   DWORD ArgCount = 0;
   LPTSTR *ArgArray = NULL;
   TCHAR szRegKey[512];
-  TCHAR szValue[128];
-  DWORD nSize;
-  HKEY hParamKey = NULL;	/* To read startup parameters */
+  HKEY hParamKey = NULL;
   DWORD TotalParams = 0;
   DWORD i;
   InputParams ThreadInputParams;
@@ -672,7 +673,7 @@ ServiceMain (DWORD argc, LPTSTR argv[])
 
   /*
    * SCM sends Service Name as first arg, increment to point
-   * arguments user specified while starting contorl agent
+   * arguments user specified while starting control agent
    */
 
   /*
@@ -681,17 +682,17 @@ ServiceMain (DWORD argc, LPTSTR argv[])
   ArgCount = 1;
 
   /*
-   * Create Registry Key path 
+   * Create registry key path 
    */
-  _snprintf (szRegKey, sizeof(szRegKey), "%s%s\\%s",
-	     _T ("SYSTEM\\CurrentControlSet\\Services\\"), app_name,
-	     "Parameters");
+  _sntprintf (szRegKey, CountOf(szRegKey), _T("%s%s\\%s"),
+	     _T ("SYSTEM\\CurrentControlSet\\Services\\"), app_name_long,
+	     _T("Parameters"));
   if (RegOpenKeyEx
       (HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_ALL_ACCESS, &hParamKey) == ERROR_SUCCESS)
     {
 
       /*
-       * Read startup Configuration information 
+       * Read startup configuration information 
        */
       /*
        * Find number of subkeys inside parameters 
@@ -707,8 +708,8 @@ ServiceMain (DWORD argc, LPTSTR argv[])
 	      /*
 	       * Allocate memory to hold strings 
 	       */
-	      ArgArray = (LPTSTR *) malloc (sizeof (LPTSTR) * ArgCount);
-	      if (ArgArray == 0)
+	      ArgArray = calloc(ArgCount, sizeof(ArgArray[0]));
+              if (ArgArray == 0)
                 {
                   WriteToEventLog (EVENTLOG_ERROR_TYPE,
 		       _T ("Resource failure"));
@@ -721,19 +722,46 @@ ServiceMain (DWORD argc, LPTSTR argv[])
 	      ArgArray[0] = _tcsdup (argv[0]);
 	      for (i = 1; i <= TotalParams; i++)
 		{
+                  DWORD dwErrorcode;
+                  DWORD nSize;
+                  DWORD nRegkeyType;
+                  TCHAR *szValue;
 
 		  /*
 		   * Create Subkey value name 
 		   */
-		  _snprintf (szRegKey, sizeof(szRegKey), "%s%d", "Param", i);
+		  _sntprintf (szRegKey, CountOf(szRegKey), _T("%s%d"), _T("Param"), i);
 
 		  /*
-		   * Set size 
+		   * Query subkey.
 		   */
-		  nSize = 128;
-		  RegQueryValueEx (hParamKey, szRegKey, 0, NULL,
-				   (LPBYTE) & szValue, &nSize);
-		  ArgArray[i] = _tcsdup (szValue);
+		  nSize = 0;
+		  dwErrorcode = RegQueryValueEx(hParamKey, szRegKey, NULL,
+                                                &nRegkeyType, NULL, &nSize);
+                  if (dwErrorcode == ERROR_SUCCESS) {
+                    if (nRegkeyType == REG_SZ || nRegkeyType == REG_EXPAND_SZ) {
+                      szValue = malloc(nSize + sizeof(szValue[0]));
+                      if (szValue) {
+		        dwErrorcode = RegQueryValueEx(hParamKey, szRegKey, NULL,
+                                                      &nRegkeyType, (LPBYTE)szValue, &nSize);
+                        if (dwErrorcode == ERROR_SUCCESS) {
+                          szValue[nSize] = 0;
+                          ArgArray[i] = szValue;
+                        } else {
+                          free(szValue);
+                          WriteToEventLog(EVENTLOG_ERROR_TYPE, _T("Querying registry key %s failed: error code %ld"), szRegKey, dwErrorcode);
+                        }
+                      } else
+                        WriteToEventLog(EVENTLOG_ERROR_TYPE, _T("Querying registry key %s failed: out of memory"), szRegKey);
+                    } else
+                      WriteToEventLog(EVENTLOG_ERROR_TYPE, _T("Type %ld of registry key %s is incorrect"), nRegkeyType, szRegKey);
+                  } else
+                    WriteToEventLog(EVENTLOG_ERROR_TYPE, _T("Querying registry key %s failed: error code %ld"), szRegKey, dwErrorcode);
+
+                  if (!ArgArray[i]) {
+                    TotalParams = ArgCount = i;
+                    break;
+                  }
 		}
 	    }
 	}
@@ -743,7 +771,7 @@ ServiceMain (DWORD argc, LPTSTR argv[])
     {
 
       /*
-       * No statup agrs are given 
+       * No startup args are given 
        */
       ThreadInputParams.Argc = argc;
       ThreadInputParams.Argv = argv;
@@ -758,7 +786,7 @@ ServiceMain (DWORD argc, LPTSTR argv[])
   /*
    * Register Service Control Handler 
    */
-  hServiceStatus = RegisterServiceCtrlHandler (app_name, ControlHandler);
+  hServiceStatus = RegisterServiceCtrlHandler (app_name_long, ControlHandler);
   if (hServiceStatus == 0)
     {
       WriteToEventLog (EVENTLOG_ERROR_TYPE,
@@ -767,12 +795,12 @@ ServiceMain (DWORD argc, LPTSTR argv[])
     }
 
   /*
-   * Update the service status to START_PENDING 
+   * Update the service status to START_PENDING.
    */
   UpdateServiceStatus (SERVICE_START_PENDING, NO_ERROR, SCM_WAIT_INTERVAL);
 
   /*
-   * Spin of worker thread, which does majority of the work 
+   * Start the worker thread, which does the majority of the work .
    */
   TRY
   {
@@ -785,7 +813,7 @@ ServiceMain (DWORD argc, LPTSTR argv[])
     hServiceThread =
       (void *) _beginthreadex (&SecurityAttributes, 0,
 			       ThreadFunction,
-			       (void *) &ThreadInputParams, 0, &dwThreadId);
+			       (void *) &ThreadInputParams, 0, &threadId);
     if (hServiceThread == NULL)
       {
 	WriteToEventLog (EVENTLOG_ERROR_TYPE, _T ("Couldn't start worker thread"));
@@ -793,13 +821,12 @@ ServiceMain (DWORD argc, LPTSTR argv[])
       }
 
     /*
-     * Set Service Status to Running 
+     * Set service status to SERVICE_RUNNING.
      */
     UpdateServiceStatus (SERVICE_RUNNING, NO_ERROR, SCM_WAIT_INTERVAL);
 
     /*
-     * Wait for termination event and worker thread to
-     * * spin down.
+     * Wait until the worker thread finishes.
      */
     WaitForSingleObject (hServiceThread, INFINITE);
   }
@@ -814,12 +841,12 @@ ServiceMain (DWORD argc, LPTSTR argv[])
     FreeSecurityAttributes (&SecurityAttributes);
 
     /*
-     * Delete allocated argument list 
+     * Free allocated argument list 
      */
     if (ArgCount > 1 && ArgArray != NULL)
       {
 	/*
-	 * Delete all strings 
+	 * Free all strings 
 	 */
 	for (i = 0; i < ArgCount; i++)
 	  {
@@ -852,7 +879,7 @@ RunAsService (INT (*ServiceFunction) (INT, LPTSTR *))
   /*
    * Initialize ServiceTableEntry table 
    */
-  ServiceTableEntry[0].lpServiceName = app_name;	/* Application Name */
+  ServiceTableEntry[0].lpServiceName = app_name_long;	/* Application Name */
 
   /*
    * Call SCM via StartServiceCtrlDispatcher to run as Service 
@@ -866,7 +893,7 @@ RunAsService (INT (*ServiceFunction) (INT, LPTSTR *))
        * Some other error has occurred. 
        */
       WriteToEventLog (EVENTLOG_ERROR_TYPE,
-		       _T ("Couldn't start service - %s"), app_name);
+		       _T ("Couldn't start service - %s"), app_name_long);
     }
   return g_fRunningAsService;
 }
@@ -998,7 +1025,7 @@ FreeSecurityAttributes (SECURITY_ATTRIBUTES * pSecurityAttr)
      * Input:
      *   lpParam contains argc and argv, pass to service main function 
      */
-DWORD WINAPI
+unsigned WINAPI
 ThreadFunction (LPVOID lpParam)
 {
   InputParams * pInputArg = (InputParams *) lpParam;
